#ifndef _DIS_COMMON_H_
#define _DIS_COMMON_H_

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /* __cpluscplus */
#endif /* __cpluscplus */
    
#include "dis.h"
#include "disosadapt.h"
#include "apr_ring.h"

#define MAX_SEND_INFO  5471
#define MAX_NODE  32
#define BASE64MODE 0
#define PROTOBUFFMODE 1
#define STRING_APPEND(_buffer, _len, _bufferSize, format, ...)  _len += snprintf_s(&(_buffer[_len]), (_bufferSize-_len), format, __VA_ARGS__)
#define MAX_LINE_LEN 1024


typedef struct _DisRecordListItem
{
    APR_RING_ENTRY(_DisRecordListItem) RecordListLink;
    DISString  RawData;
    char       partitionKey[256];
    char       explicitHashKey[16];
    char       partitionId[64]; 
    long       RspCode;
    char       seqNumber[16];
    char       ErrMsg[128];
    DISPutRecordCallback  pfNotify;
    char       data[0];
}DisRecordListItem;

typedef struct _DisRecordList
{
    long volatile InitFlag;
    long  CurrentCount;
    long  MaxCount;
    long  AllInputCount;
    long  AllOutputCount;
    SM_T  mutex;
    APR_RING_HEAD(_DisRecordListHead, _DisRecordListItem) list;
}DisRecordList;

typedef struct _SendInfo
{
    char   Endpoint[MAX_ENDPOINT_LEN];
    char   StreamName[64+8];
    char   ProjectId[MAX_PROJECTID_LEN];
    char   Region[MAX_REGION_LEN];
}SendInfo;

typedef struct _SendInfoHashItem
{
    long nHashA;
    long nHashB;
    long bExists;
    long reserve;
    SendInfo  SendInfoDetail;
    DisRecordList *RecordList;
}SendInfoHashItem;

typedef struct _SendInfoHashTable
{
    long volatile InitFlag;
    long MaxCount;
    long CurrentCount;
    SM_T mutex;
    SendInfoHashItem  *SendInfoItemList;
}SendInfoHashTable;

int  DisSendInfoHashTableInit(int MemCareFlag);
void DisSendInfoHashTableDeInit();
int  DisGetSendRecordList(char *Endpoint, char *ProjectId, char *Region, char *StreamName, DisRecordList **RecordList);
void DisHashTableDbgInfo();

typedef struct _DisContext
{
    char   *ak;
    char   *sk;
    char   *projectId;
    char   *hostname;
    char   *region;
}DISContext;

DISStatus GetUserAuthInfoFromPropertyFile(char *ProjectId, char *akArray, char *skArray, char *xSecurityToken);

int DisRecordListInit(int MaxNum, DisRecordList **list);

int DisRecordListDeinit(DisRecordList *list);

DisRecordListItem* DisRecordListItemAlloc(DISPutRecord *data);

void DisRecordListItemRelease(DisRecordListItem *node);

int DisRecordListInsertToHead(DisRecordList *list, DisRecordListItem *node);

int DisRecordListInsertToTail(DisRecordList *list, DisRecordListItem *node);

DisRecordListItem* DisRecordListGetNode(DisRecordList *list); 

void DisListDebug(DisRecordList *list);

int DisCheckPutRecord(DISPutRecord *putRecord);

extern SendInfoHashTable g_stSendInfoTable;
extern int g_InitFlag;
extern DISGetUserAuthInfp   g_GetUserAuthInfo;
extern int g_SerializedMode;


#ifdef __cplusplus
#if __cplusplus
}
#endif /* __cpluscplus */
#endif /* __cpluscplus */

#endif