/** **************************************************************************
 * util.h
 * 
 * Copyright 2008 Bryan Ischo <bryan@ischo.com>
 * 
 * This file is part of libs3.
 * 
 * libs3 is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, version 3 of the License.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of this library and its programs with the
 * OpenSSL library, and distribute linked combinations including the two.
 *
 * libs3 is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with libs3, in a file named COPYING.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 ************************************************************************** **/

#ifndef _DIS_UTIL_H
#define _DIS_UTIL_H


#include <stdint.h>
#include "dis.h"

void memset_S(void* dest, size_t destMax, int c, size_t count);
void memcpy_S(void* dest, size_t destMax, const void* src, size_t count);

unsigned int base64_encode(unsigned char *sourceStr, unsigned char* Base64Str, int sourcelen) ;
unsigned int base64_decode(unsigned char *sourceStr, unsigned char *binStr);

// Utilities -----------------------------------------------------------------

// Compute HMAC-SHA-1 with key [key] and message [message], storing result
// in [hmac]
void HMAC_SHA1(unsigned char hmac[20], const unsigned char *key, int key_len,
               const unsigned char *message, int message_len);

//  Compute HMAC-SHA-256 with key [key] and message [message], storing result in [hmac]
//  The string's length is 32 bytes,and the string could be transformed to 16 Hexadecimal when it is used.
void HMAC_SHA256(unsigned char hmac[32], const unsigned char *key, int key_len,
               const unsigned char *message, int message_len);

//  COMPUTE SHA256Hash with message [message], stoing result in [sha]
//  The string's length is 32 bytes,and the string could be transformed to 16 Hexadecimal when it is used.
void SHA256Hash(unsigned char sha[32], const unsigned char *message, int message_len);

// Compute a 64-bit hash values given a set of bytes
uint64_t hash(const unsigned char *k, int length);

// Because Windows seems to be missing isblank(), use our own; it's a very
// easy function to write in any case
int is_blank(char c);

void ustr_to_hexes(unsigned char* szIn,unsigned int inlen, unsigned char* szOut);

#endif /* UTIL_H */
