#include "stdio.h"
#include "string.h"
#include "log.h"
#include "dismemmgt.h"
#include "dis.h"
#include "cJSON.h"
#include "disrequest.h"
#include "discommon.h"
#include "curl/curl.h"
#include "disnetwork.h"
#include "disbodyencode.h"
#include "discommon.h"
#include "util.h"
#include "httpbody.pb-c.h"
#include "properties.h"


char g_Version[MAX_VERSION_INFO_LEN] = { 0 };

int versionInit()
{
    int ret = 0;
    char platfrom[10] = {0};
    char version[10] = {0};
    
    ret = read_properties(DISVERSIONINFOFILEPATH, "platform", platfrom);
    if(0 != ret)
    {
      printf("---%d----",ret);
      return DISStatusVersionInitError;
    }

    ret = read_properties(DISVERSIONINFOFILEPATH, "version", version);
    if(0 != ret)
    {
      printf("---%d---",ret);
      return DISStatusVersionInitError;
    }
    strncpy(g_Version, version, strlen(version));
  	strncat(g_Version, "/", 1);
  	strncat(g_Version, platfrom, strlen(platfrom));
 
    //replcae g_Version \r with \0
    if (g_Version[strlen(g_Version)-1] == '\r')
    {
      g_Version[strlen(g_Version)-1] = '\0'; 
    }
    else if (g_Version[strlen(g_Version)-1] == '\n')
    {
      g_Version[strlen(g_Version)-1] = '\0'; 
    }
    
    printf("-------%s-------\n",g_Version);
    return ret;
}


DISStatus DisInit(FILE *logFile, DISGetUserAuthInfp handle)
{
    int iRet = 0;
    cJSON_Hooks stHook = {0};

    printfdbgfunc;
    
   	// init mutex
    iRet = pthread_mutex_init(&mutex, NULL);
  	if (0 != iRet) {
  		// init mutex fail
  		return DISStatusVersionInitError;
  	}

    iRet = versionInit();
    if(0 != iRet)
    {
        return DISStatusVersionInitError;
    }

    if(NULL == logFile)
    {
        return DISStatusInvalidParam;
    }

    if(0 != g_InitFlag)
    {
        return 0;
    }

    g_InitFlag = 1;

	logPrint(DISLogLevelInfo,"init the logfile\r\n");
    iRet = setLogFile(logFile);
    if (0 !=  iRet)
    {
        return DISStatusInternalError;
    }

	logPrint(DISLogLevelInfo,"init mem start\r\n");
    logPrint(DISLogLevelALL, "init start");
    iRet = disMemPoolInit();
    if(0 != iRet)
    {
        logPrint(DISLogLevelALL, "Mem init error : %d\r\n", iRet);
        DisDeinit();
        return DISStatusInternalError;
    }

	logPrint(DISLogLevelALL, "init Hash Table");
    iRet = DisSendInfoHashTableInit(1);
    if(0 != iRet)
    {
        logPrint(DISLogLevelALL, "HashTable init error : %d\r\n", iRet);
        DisDeinit();
        return DISStatusInternalError;
    }

	logPrint(DISLogLevelALL, "init newwork");
    iRet = DisNetworkInit();
    if(0 != iRet)
    {
        logPrint(DISLogLevelALL, "Network init error : %d\r\n", iRet);
        DisDeinit();
        return DISStatusInternalError;
    }

    if(handle)
    {
        g_GetUserAuthInfo = handle;
    }
    else
    {
        g_GetUserAuthInfo = GetUserAuthInfoFromPropertyFile;
    }
    
    stHook.malloc_fn = disMemAlloc;
    stHook.free_fn = disMemFree;

    cJSON_InitHooks(&stHook);

    logPrint(DISLogLevelALL, "init success");
    return DISStatusOK;
}

void DisDeinit()
{
    logPrint(DISLogLevelALL, "Dis Deinit");
    if( 0 == g_InitFlag)
    {
        return ;
    }

    g_InitFlag = 0;

    DisNetworkDeinit();
    DisSendInfoHashTableDeInit();
    disMemPoolDeinit(); 
	//reinit mutex
	pthread_mutex_destroy(&mutex);
    return ;
}

DISStatus DISSetSerializedMode(char* serializedMode)
{
    if(0 == strcmp(serializedMode,"protobuf"))
    {
        g_SerializedMode = 2 ;
        return DISStatusOK;
    }
    else if(0 == strcmp(serializedMode,"base64"))
    {
        g_SerializedMode = 1 ;        
        return DISStatusOK;
    }
    else
    {
        logPrint(DISLogLevelError, "The input param is invalid, serializedMode:%s", serializedMode);
    }
    return DISStatusOK;
}


DISStatus CreateStream
(
    char             *Endpoint,
    char             *ProjectId,
    char             *Region,
    DISCreateStream  *streamInfo,
    DISResponseInfo  *ResponseInfo
)
{
    DISStatus ret;
    DisRequest *pstTempRequest = NULL;
    int len = 0;
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    DISContext  disCtx = {0};

    printfdbgfunc;

	logPrint(DISLogLevelInfo, "Create Stream Start");

    if (g_InitFlag == 0)
    {
		logPrint(DISLogLevelError, "The dissdk is not init");
        return DISStatusNotInit;
    }

    if((NULL == streamInfo) || (NULL == Endpoint) | (NULL == ProjectId) || (NULL == Region) || (NULL == ResponseInfo))
    {
		logPrint(DISLogLevelError, "The input param is invalid, streamInfo:%d, Endpoint:%d, ProjectId:%d, Region:%d, ResponseInfo:%d", streamInfo, Endpoint, ProjectId, Region, ResponseInfo);
        return DISStatusInvalidParam;
    }

    ret = DisValidateDisString(streamInfo->streamName, strlen(streamInfo->streamName), MAX_STREAMNAME_LEN);
    if(DISStatusOK != ret)
    {
		logPrint(DISLogLevelError, "The input streamname invalid:%d", ret);
        return ret;
    }

	logPrint(DISLogLevelInfo, "streamName:%s, Endpoint:%s, ProjectId:%s, Region:%s", streamInfo->streamName, Endpoint, ProjectId, Region);

    if(streamInfo->streamType != DISStreamTypeCommon && streamInfo->streamType !=DISStreamTypeAdvanced)
    {
		logPrint(DISLogLevelError, "The input streamType invalid:%d", streamInfo->streamType);
        return DISStatusInvalidStreamType;
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
		logPrint(DISLogLevelFatal, "memory alloc :%d error", MAX_X_SECURITY_TOKEN_LEN);
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
		logPrint(DISLogLevelError, "get user auth info error: %d", ret);
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    len = strlen(streamInfo->streamName) + BASE_BODY_LEN;

    ret = DisGetRequest(&pstTempRequest, DISInterfaceCreateStream, &disCtx, len);
    if(DISStatusOK != ret)
    {
		logPrint(DISLogLevelError, "get request error: %d", ret);
        disMemFree(xSecurityToken);
        return ret;
    }

    ret = DisComposeUri(pstTempRequest, &disCtx, streamInfo->streamName);
    if(DISStatusOK != ret)
    {
		logPrint(DISLogLevelError, "Compose Uri error: %d", ret);
        disMemFree(xSecurityToken);
        DisReleaseRequest(pstTempRequest);
        return ret;
    }

    ret = DisBodyEncodeStream(streamInfo, pstTempRequest->request.body, &(pstTempRequest->request.bodyCurrentLen), pstTempRequest->request.bodyMaxLen);
    if(DISStatusOK != ret)
    {
		logPrint(DISLogLevelError, "Create Stream Encode Body Error: %d", ret);
        disMemFree(xSecurityToken);
        DisReleaseRequest(pstTempRequest);
        return ret;
    }

    ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
    printf("ret %d\n",ret);
   	if (DISStatusOK != ret)
  	{
  		logPrint(DISLogLevelError, "Create Stream Failed: %d", ret);
  		disMemFree(xSecurityToken);
  		DisReleaseRequest(pstTempRequest);
  		return ret;
  	}
    disMemFree(xSecurityToken);
    DisReleaseRequest(pstTempRequest);

    return ret;
}

DISStatus DeleteStream
(
        char             *Endpoint,
        char             *ProjectId,
        char             *Region,
        char             *streamName,
        DISResponseInfo  *ResponseInfo
)
{
    DISStatus ret = DISStatusOK;
    DisRequest *pstTempRequest = NULL;
    //int len = 0;
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    DISContext  disCtx = {0};

    printfdbgfunc;

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }

    if((NULL == streamName) || (NULL == Endpoint) | (NULL == ProjectId) || (NULL == Region) || (NULL == ResponseInfo))
    {
        return DISStatusInvalidParam;
    }


    ret = DisValidateDisString(streamName, strlen(streamName), MAX_STREAMNAME_LEN);
    if(DISStatusOK != ret)
    {
        return ret;
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    ret = DisGetRequest(&pstTempRequest, DISInterfaceDeleteStream, &disCtx, 0);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        return ret;
    }

    ret = DisComposeUri(pstTempRequest, &disCtx, streamName);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        DisReleaseRequest(pstTempRequest);
        return ret;
    }

    ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        DisReleaseRequest(pstTempRequest);
        return ret;
    }
    DisReleaseRequest(pstTempRequest);
    disMemFree(xSecurityToken);

    return DISStatusOK;
}

DISStatus DescribeStream
(
        char              *Endpoint,
        char              *ProjectId,
        char              *Region,
        DISDescribeStream *streamDetail,
        DISResponseInfo   *ResponseInfo
)
{
    DISStatus ret;
    char partitionNum[12] = {0};
    //int len = 0;
    DisRequest *pstTempRequest = NULL;
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    DISContext  disCtx = {0};
	int Count = 0;
	cJSON *BodyData = NULL;
	cJSON *streamId;
	cJSON *streamName;
	cJSON *createTime;
	cJSON *lastModifyTime;
	cJSON *retentionPeriod;
	cJSON *streamStatus;
	cJSON *streamType;
	cJSON *hasMorepartition;
	cJSON *partitionList;
	cJSON *partition;
	cJSON *dataType;
    printfdbgfunc;

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }

    if((NULL == streamDetail) || (NULL == Endpoint) | (NULL == ProjectId) || (NULL == Region) || (NULL == ResponseInfo))
    {
        return DISStatusInvalidParam;
    }

    ret = DisValidateDisString(streamDetail->streamName, strlen(streamDetail->streamName), MAX_STREAMNAME_LEN);
    if(DISStatusOK != ret)
    {
        return ret;
    }

    if(streamDetail->startPartitionId)
    {
        ret = DisValidateDisString(streamDetail->startPartitionId, strlen(streamDetail->startPartitionId), MAX_PARTITIONID_LEN);
        if(DISStatusOK != ret)
        {
            return ret;
        }
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    ret = DisGetRequest(&pstTempRequest, DISInterfaceDescribeStream, &disCtx, 0);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        return ret;
    }

    do{
        ret = DisComposeUri(pstTempRequest, &disCtx, streamDetail->streamName);
        if(DISStatusOK != ret)
        {
            break;
        }

        if(NULL != streamDetail->startPartitionId)
        {
            ret = DisAddQueryList(pstTempRequest, "start_partitionId", streamDetail->startPartitionId);
            if(DISStatusOK != ret)
            {
                break;
            }
        }

        if(0 != streamDetail->limitPartitionNum)
        {
            //itoa(streamDetail->limitPartitionNum, partitionNum, 10);
            sprintf(partitionNum, "%d", (int)(streamDetail->limitPartitionNum));
            ret = DisAddQueryList(pstTempRequest, "limit_partitions", partitionNum);
            if(DISStatusOK != ret)
            {
                break;
            }
        }

    }while(0);

    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        DisReleaseRequest(pstTempRequest);
        return ret;
    }   

	do
	{
		ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
		if (DISStatusOK == ret && pstTempRequest->httpResponseCode >= 200 && pstTempRequest->httpResponseCode < 300)
		{
			BodyData = cJSON_Parse(pstTempRequest->RspBody);
			pstTempRequest->jsonData = BodyData;

			streamId = cJSON_GetObjectItem(BodyData, "stream_id");
			streamName = cJSON_GetObjectItem(BodyData, "stream_name");
			createTime = cJSON_GetObjectItem(BodyData, "create_time");
			lastModifyTime = cJSON_GetObjectItem(BodyData, "last_modified_time");
			retentionPeriod = cJSON_GetObjectItem(BodyData, "retention_period");
			streamStatus = cJSON_GetObjectItem(BodyData, "status");
			streamType = cJSON_GetObjectItem(BodyData, "stream_type");
			hasMorepartition = cJSON_GetObjectItem(BodyData, "has_more_partitions");
			partitionList = cJSON_GetObjectItem(BodyData, "partitions");
			dataType = cJSON_GetObjectItem(BodyData, "data_type");

			if (NULL == streamId || NULL == streamName || NULL == createTime || NULL == lastModifyTime
				|| NULL == retentionPeriod || NULL == streamStatus || NULL == streamType
				|| NULL == hasMorepartition || NULL == partitionList)
			{
				logPrint(DISLogLevelInfo, "the body is not correct: %d, %s\r\n", __LINE__, pstTempRequest->RspBody);
				ret = DISStatusInvalidResponse;
				break;
			}
			
			int array_size = cJSON_GetArraySize(partitionList);
			for (Count = 0; Count < array_size; Count++)
			{
				partition = cJSON_GetArrayItem(partitionList, Count);
				if (NULL == partition)
				{
				logPrint(DISLogLevelInfo, "the body is not correct: %d, %s\r\n", Count, pstTempRequest->RspBody);
				break;
				}

				cJSON *partitionStatus = cJSON_GetObjectItem(partition, "status");
				cJSON *partitionId = cJSON_GetObjectItem(partition, "partition_id");
				cJSON *hashRange = cJSON_GetObjectItem(partition, "hash_range");
				cJSON *sequenceNumberRange = cJSON_GetObjectItem(partition, "sequence_number_range");

				char  hashRangeStr[1024];
				char  sequenceNumberRangeStr[1024];
				char * startHashIndex;
				char * endHashIndex;
				char * startSeqNumber;
				char * endSeqNumber;
				if (NULL == partitionStatus || NULL == partitionId || NULL == hashRange || NULL == sequenceNumberRange)
				{
					logPrint(DISLogLevelInfo, "the body is not correct: %d, %s\r\n", __LINE__, pstTempRequest->RspBody);
					ret = DISStatusInvalidResponse;
					break;
				}

				streamDetail->streamInfo.partitionList[Count].partitionStatus = (strcmp(partitionStatus->valuestring, "CREATING") == 0) ? DISPartitionStatusCreating
					: (strcmp(partitionStatus->valuestring, "ACTIVE") == 0) ? DISPartitionStatusActive
					: (strcmp(partitionStatus->valuestring, "DELETED") == 0) ? DISPartitionStatusDeleted
					: DISPartitionStatusExpired;
				strcpy((streamDetail->streamInfo.partitionList[Count].partitionId), partitionId->valuestring);
				strcpy((streamDetail->streamInfo.partitionList[Count].hashRange), hashRange->valuestring);
				strcpy((streamDetail->streamInfo.partitionList[Count].sequenceNumberRange), sequenceNumberRange->valuestring);
			}
			
			streamDetail->streamInfo.streamId = (char *)malloc(sizeof(strlen(streamId->valuestring)+1));
			streamDetail->streamInfo.streamName = (char *)malloc(sizeof(strlen(streamName->valuestring) + 1));
			strcpy((streamDetail->streamInfo.streamId), streamId->valuestring);
			strcpy((streamDetail->streamInfo.streamName), streamName->valuestring);
			streamDetail->streamInfo.createTime = createTime->valuedouble;
			streamDetail->streamInfo.lastModifyTime = lastModifyTime->valuedouble;
			streamDetail->streamInfo.retentionPeriod = retentionPeriod->valuedouble;
			streamDetail->streamInfo.streamStatus = (strcmp(streamStatus->valuestring, "CREATING") == 0) ? DISStreamStatusCreating
				: (strcmp(streamStatus->valuestring, "RUNNING") == 0) ? DISStreamStatusRunning
				: (strcmp(streamStatus->valuestring, "TERMINATING") == 0) ? DISStreamStatusTerminating
				: DISStreamStatusFrozen;
			streamDetail->streamInfo.streamType =  (strcmp(streamType->valuestring, "ADVANCED")==0) ? DISStreamTypeAdvanced : DISStreamTypeCommon;
			streamDetail->streamInfo.hasMorepartition = hasMorepartition->valueint;
			streamDetail->streamInfo.currentNumber = array_size;
			streamDetail->streamInfo.dataType = (strcmp(dataType->valuestring, "BLOB") == 0) ? DISDataTypeBlob
				: (strcmp(dataType->valuestring, "JSON") == 0) ? DISDataTypeJson
				: (strcmp(dataType->valuestring, "CSV") == 0) ? DISDataTypeCsv
				: DISDataTypeFile;

		}
	} while (0);

    DisReleaseRequest(pstTempRequest);
    disMemFree(xSecurityToken);

    return ret;
}

DISStatus ListStream
(
        char             *Endpoint,
        char             *ProjectId,
        char             *Region,
        DISListStream    *streamList,
        DISResponseInfo  *ResponseInfo
)
{
    DISStatus ret = 0;
    DisRequest *pstTempRequest = NULL;
    char streamNum[12] = {0};
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    DISContext  disCtx = {0};
    cJSON *BodyData = NULL;
    cJSON *StreamArray = NULL;
    cJSON *TotalNumber = NULL;
    cJSON *HaveMoreFlag = NULL;
    cJSON *StreamInfo = NULL;
    int Count = 0;

    printfdbgfunc;  

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }

    if((NULL == streamList) || (NULL == Endpoint) | (NULL == ProjectId) || (NULL == Region) || (NULL == ResponseInfo))
    {
        return DISStatusInvalidParam;
    }

    if(streamList->startStreamName)
    {
        ret = DisValidateDisString(streamList->startStreamName, strlen(streamList->startStreamName), MAX_STREAMNAME_LEN);
        if(DISStatusOK != ret)
        {
            return ret;
        }
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    ret = DisGetRequest(&pstTempRequest, DISInterfaceListStream, &disCtx, 0);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        return ret;
    }

    do
    {
        ret = DisComposeUri(pstTempRequest, &disCtx, NULL);
        if(DISStatusOK != ret)
        {
            break;
        }

        if(0 != streamList->limit)
        {
            //itoa(streamList->limit, streamNum, 10);
            sprintf(streamNum, "%d", (int)(streamList->limit));
            ret = DisAddQueryList(pstTempRequest, "limit", streamNum);
            if(DISStatusOK != ret)
            {
                break;
            }
        }

        if(NULL != streamList->startStreamName)
        {
            ret = DisAddQueryList(pstTempRequest, "start_stream_name", streamList->startStreamName);
            if(DISStatusOK != ret)
            {
                break;
            }
        }

        ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
        if(DISStatusOK == ret && pstTempRequest->httpResponseCode >=200 && pstTempRequest->httpResponseCode < 300)
        {
            BodyData = cJSON_Parse(pstTempRequest->RspBody);
            pstTempRequest->jsonData = BodyData;

            StreamArray = cJSON_GetObjectItem(BodyData, "stream_names");
            TotalNumber = cJSON_GetObjectItem(BodyData, "total_number");
            HaveMoreFlag = cJSON_GetObjectItem(BodyData, "has_more_streams");
            if(NULL == StreamArray || NULL == TotalNumber || NULL == HaveMoreFlag)
            {
                logPrint(DISLogLevelInfo,"the body is not correct: %d, %s\r\n", __LINE__, pstTempRequest->RspBody);
                ret = DISStatusInvalidResponse;
                break;
            }

            for(Count = 0; Count < TotalNumber->valueint; Count++)
            {
                StreamInfo = cJSON_GetArrayItem(StreamArray, Count);
                if(NULL == StreamInfo || NULL == StreamInfo->valuestring)
                {
                    logPrint(DISLogLevelInfo,"the body is not correct: %d, %s\r\n", Count, pstTempRequest->RspBody);
                    break;
                }

                strncpy((streamList->streamListResult.streamList[Count].streamName), StreamInfo->valuestring, strlen(StreamInfo->valuestring));
            }
            streamList->streamListResult.currentNumber = Count;
            streamList->streamListResult.totalNumber = TotalNumber->valueint;

            streamList->streamListResult.hasMoreStream = HaveMoreFlag->valueint;

        }

    }while(0);

    disMemFree(xSecurityToken);
    DisReleaseRequest(pstTempRequest);

    return ret;
}


DISStatus PutRecordsAsyn
(
        char                   *Endpoint,
        char                   *ProjectId,
        char                   *Region,
        char                   *streamName,
        DISPutRecord           *putRecord,
        DISPutRecordCallback    handle
)
{
    DISStatus ret = 0;
    DisRecordList *list = NULL;
    DisRecordListItem *node = NULL;

    printfdbgfunc;  

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }

    if(NULL == Endpoint || NULL == streamName || NULL == putRecord || NULL == ProjectId || NULL == Region)
    {
        return DISStatusInvalidParam;
    }

    if((strlen(Endpoint) >= MAX_ENDPOINT_LEN) || strlen(streamName) >= MAX_STREAMNAME_LEN || strlen(Region) >= MAX_REGION_LEN || strlen(ProjectId) >= MAX_PROJECTID_LEN)
    {
        return DISStatusInvalidParam;
    }
#if 1
    ret = DisGetSendRecordList(Endpoint, ProjectId, Region, streamName, &list);
    if(0 != ret || NULL == list)
    {
        return DISStatusInternalError;
    }

    node = DisRecordListItemAlloc(putRecord);
    if(NULL == node)
    {
        return DISStatusNoMemory;
    }

    node->pfNotify = handle;

    ret = DisRecordListInsertToTail(list, node);
    if(0 != ret)
    {
        DisRecordListItemRelease(node);
        return ret;
    }
#endif
    printfdbgoutfunc;
    return DISStatusOK;
}

DISStatus PutRecords
(
        char                   *Endpoint,
        char                   *ProjectId,
        char                   *Region,
        char                   *streamName,
        int                     Count,
        DISPutRecord           *putRecord,
        DISPutRecordCallback    handle,
        DISResponseInfo        *ResponseInfo
)
{
    DISStatus ret = 0;
    DisRequest *pstTempRequest = NULL;
    //char streamNum[12] = {0};
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    DISContext  disCtx = {0};
    int BodyLen = 0;
    int Loop = 0;

    cJSON  *BodyData = NULL;
    cJSON  *Record = NULL;
    cJSON  *RecordArray = NULL;
	cJSON  *FailedRecordCount = NULL;
    //int     RecordCount = 0;
    cJSON  *SeqNumber = NULL;
    cJSON  *partitionId = NULL;
    cJSON  *RecordRspString = NULL;
    cJSON  *RecordRspErrInfo = NULL;
    char   *RecordRspCodeString = NULL;
    char   *RecordRspErrInfoString = NULL;
    int serrializedMode = DISGetSerializedMode();

	printfdbgfunc;  

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }
    
    if(NULL == Endpoint || NULL == streamName || NULL == putRecord || NULL == ProjectId || NULL == Region || 0 == Count || NULL == ResponseInfo)
    {
        return DISStatusInvalidParam;
    }

    if((strlen(Endpoint) >= MAX_ENDPOINT_LEN) || strlen(streamName) >= MAX_STREAMNAME_LEN || strlen(Region) >= MAX_REGION_LEN || strlen(ProjectId) >= MAX_PROJECTID_LEN)
    {
        return DISStatusInvalidParam;
    }

	if (Count > MAX_RECORD_COUNT)
	{
		return DISStatusInvalidParam;
	}

    for(Loop = 0; Loop < Count; Loop++)
    {
        ret = DisCheckPutRecord(putRecord+Loop);
        if(0 != ret)
        {
            return DISStatusInvalidParam;
        }

        BodyLen += (putRecord + Loop)->recordData.stringLen;
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    BodyLen = BodyLen * 1.6 + BASE_BODY_LEN + (sizeof(DISPutRecord)*Count);

    ret = DisGetRequest(&pstTempRequest, DISInterfacePutRecords, &disCtx, BodyLen);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        return ret;
    }

    do
    {
        ret = DisComposeUri(pstTempRequest, &disCtx, NULL);
        if(DISStatusOK != ret)
        {
            break;
        }
        
        if(BASE64MODE == serrializedMode)
        {
            ret = (DISStatus)DisBodyEncodeRecords(streamName, putRecord, Count, pstTempRequest->request.body, &(pstTempRequest->request.bodyCurrentLen),pstTempRequest->request.bodyMaxLen);
        }
        else if(PROTOBUFFMODE == serrializedMode)
        {
            ret = (DISStatus)DisBodyEncodeRecordsByProtoBuf(streamName, putRecord, Count, pstTempRequest->request.body, &(pstTempRequest->request.bodyCurrentLen),pstTempRequest->request.bodyMaxLen);
        }
        else
        {
            break;
        }
        
        if(0 != ret)
        {
            break;
        }       

        ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
        if(DISStatusOK == ret && pstTempRequest->httpResponseCode >=200 && pstTempRequest->httpResponseCode < 300 && BASE64MODE == serrializedMode)
        {
            BodyData = cJSON_Parse(pstTempRequest->RspBody);
            pstTempRequest->jsonData = BodyData;

			FailedRecordCount = cJSON_GetObjectItem(BodyData, "failed_record_count");

            RecordArray = cJSON_GetObjectItem(BodyData, "records");
            if(NULL == RecordArray)
            {
                break;
            }

            for(Loop = 0; Loop < Count; Loop++)
            {
                SeqNumber = NULL;
                partitionId = NULL;
                Record = NULL;

                Record = cJSON_GetArrayItem(RecordArray, Loop);
                if(NULL == Record)
                {
                    break;
                }

                SeqNumber = cJSON_GetObjectItem(Record, "sequence_number");
                partitionId = cJSON_GetObjectItem(Record, "partition_id");
                RecordRspString = cJSON_GetObjectItem(Record, "error_code");
                RecordRspErrInfo = cJSON_GetObjectItem(Record, "error_message");

                if(NULL != SeqNumber && NULL != partitionId)
                {
                    if(handle)
                    {
                        handle(NULL, NULL, streamName, &putRecord[Loop], SeqNumber->valuestring, partitionId->valuestring);
                    }
                }
                else
                {
                        RecordRspCodeString = (NULL == RecordRspString) ? "DIS.ERROR" : RecordRspString->valuestring;
                        RecordRspErrInfoString = (NULL == RecordRspErrInfo) ? "Unknown Error": RecordRspErrInfo->valuestring;
                        if(handle)
                        {
                            handle(RecordRspCodeString,RecordRspErrInfoString, streamName, &putRecord[Loop], NULL, NULL);
                        }
                }
            }
        }
        else if(DISStatusOK == ret && pstTempRequest->httpResponseCode >=200 && pstTempRequest->httpResponseCode < 300 && PROTOBUFFMODE == serrializedMode)
        {
			int loopCount = 0;
			char *sequencenumber;
			char *shardid;
			char *errorcode;
			char *errormessage;
			int failedRecordCount;
			PutRecordsResultEntry *pstPutRecordsResultEntry;
			PutRecordsResult *pstPutRecordsResult;

			pstPutRecordsResult = put_records_result__unpack(NULL, pstTempRequest->RspBodyCurrentLen, pstTempRequest->RspBody);
			if (NULL == pstPutRecordsResult)
			{
				printf("%s:%d put_records_result__unpack failed\n", __FUNCTION__, __LINE__);
				return DISStatusError;
			}
			else
			{
				failedRecordCount = pstPutRecordsResult->failedrecordcount;
				for (loopCount = 0; loopCount < pstPutRecordsResult->n_records; loopCount++)
				{
					pstPutRecordsResultEntry = pstPutRecordsResult->records[loopCount];

					errorcode = pstPutRecordsResultEntry->errorcode;
					errormessage = pstPutRecordsResultEntry->errormessage;
					sequencenumber = pstPutRecordsResultEntry->sequencenumber;
					shardid = pstPutRecordsResultEntry->shardid;
					handle(errorcode, errormessage, streamName, &putRecord[loopCount], sequencenumber, shardid);
				}
				put_records_result__free_unpacked(pstPutRecordsResult, NULL);
			}
        }
        else
        {
            logPrint(DISLogLevelError,"When PutRecords DisSendRequest failed .ret = [%d]",ret);
            printf("When PutRecords DisSendRequest failed, it due to %s\n",pstTempRequest->RspBody);
        }

    }while(0);

    disMemFree(xSecurityToken);
    DisReleaseRequest(pstTempRequest);

    return ret;

}


char *DIsGetCursorType(DISCursorType  cursorType)
{
    switch(cursorType)
    {
    case DISCursorTypeAtSeqNumber:
        return "AT_SEQUENCE_NUMBER";
    case DISCursorTypeAfterSeqNumber:
        return "AFTER_SEQUENCE_NUMBER";
    case DISCursorTypeTrimHorizon:
        return "TRIM_HORIZON";
	case DISCursorTypeLatest:
		return  "LATEST";
    default:
        return "AT_TIMESTAMP";
    }
}

DISStatus DisGetCursor
(
        char                   *Endpoint,
        char                   *ProjectId,
        char                   *Region,
        DISGetCursor           *getCursor,
        DISResponseInfo        *ResponseInfo
)
{

    DISStatus ret = 0;
    DisRequest *pstTempRequest = NULL;
    //char streamNum[12] = {0};
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    DISContext  disCtx = {0};
    cJSON *BodyData = NULL;
    cJSON *PartitionCursor = NULL;
    int Len = 0;

    printfdbgfunc;  

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }

    if((NULL == ResponseInfo) || (NULL == getCursor) || (NULL == Endpoint) | (NULL == ProjectId) || (NULL == Region) || (NULL == getCursor->streamName) || (NULL == getCursor->partitionId) || (getCursor->cursorType > DISCursorTypeAtTimestamp))
    {
        return DISStatusInvalidParam;
    }

    if(getCursor->cursorType == DISCursorTypeAtSeqNumber || getCursor->cursorType == DISCursorTypeAfterSeqNumber)
    {
        if(NULL == getCursor->startingSequenceNumber)
        {
            return DISStatusInvalidParam;
        }
    }


    ret = DisValidateDisString(getCursor->streamName, strlen(getCursor->streamName), MAX_STREAMNAME_LEN);
    if(DISStatusOK != ret)
    {
        return ret;
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    ret = DisGetRequest(&pstTempRequest, DISInterfaceGetCursor, &disCtx, 0);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        return ret;
    }

    do
    {
        ret = DisComposeUri(pstTempRequest, &disCtx, NULL);
        if(DISStatusOK != ret)
        {
            break;
        }

        ret = DisAddQueryList(pstTempRequest, "stream-name", getCursor->streamName);
        if(DISStatusOK != ret)
        {
            break;
        }

        ret = DisAddQueryList(pstTempRequest, "partition-id", getCursor->partitionId);
        if(DISStatusOK != ret)
        {
            break;
        }

        ret = DisAddQueryList(pstTempRequest, "cursor-type", DIsGetCursorType(getCursor->cursorType));
        if(DISStatusOK != ret)
        {
            break;
        }

        if(getCursor->cursorType == DISCursorTypeAtSeqNumber || getCursor->cursorType == DISCursorTypeAfterSeqNumber)
        {
            ret = DisAddQueryList(pstTempRequest, "starting-sequence-number", getCursor->startingSequenceNumber);
            if(DISStatusOK != ret)
            {
                break;
            }
        }
		else if(getCursor->cursorType == DISCursorTypeAtTimestamp)
		{
			if (NULL == getCursor->startingSequenceNumber || 0 == strcmp("",getCursor->startingSequenceNumber))
			{
				ret = DisAddQueryList(pstTempRequest, "timestamp",  getCursor->timestamp);
				if (DISStatusOK != ret)
				{
					break;
				}
			}
			else
			{
				ret = DisAddQueryList(pstTempRequest, "starting-sequence-number", getCursor->startingSequenceNumber);
				if (DISStatusOK != ret)
				{
					break;
				}
			}
		}

        ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
        if(DISStatusOK == ret && pstTempRequest->httpResponseCode >=200 && pstTempRequest->httpResponseCode < 300)
        {
            BodyData = cJSON_Parse(pstTempRequest->RspBody);
            pstTempRequest->jsonData = BodyData;

            PartitionCursor = cJSON_GetObjectItem(BodyData, "partition_cursor");
            if(NULL == PartitionCursor || NULL == PartitionCursor->valuestring)
            {
                logPrint(DISLogLevelInfo,"the body is not correct: %d, %s\r\n", __LINE__, pstTempRequest->RspBody);
                ret = DISStatusInvalidResponse;
                break;
            }


            Len = strlen(PartitionCursor->valuestring);
            if(Len > 512)
            {
                logPrint(DISLogLevelInfo,"the PartitionCursor too long: %d, %s\r\n", Len, pstTempRequest->RspBody);
                ret = DISStatusInvalidResponse;
                break;
            }

            strncpy((getCursor->cursorResult.partitionCursor), PartitionCursor->valuestring, Len);
        }

    }while(0);

    disMemFree(xSecurityToken);
    DisReleaseRequest(pstTempRequest);

    return ret;
}


DISStatus GetRecords
(
        char                  *Endpoint,
        char                  *ProjectId,
        char                  *Region,
        char                  *streamName,
        char                  *partitionCursor,
        unsigned long          limit,
        DISCursor             *NextCursor,
        DISGetRecordsCallback  handle,
        DISResponseInfo       *ResponseInfo
)
{

    DISStatus ret = 0;
    DisRequest *pstTempRequest = NULL;
    char RecordLimit[12] = {0};
    char ak[64] = {0};
    char sk[128] = {0};
    char *xSecurityToken = NULL;
    char *Base64DecodeBuffer = NULL;
    int   BufferLen = 1024*1024;
    DISContext  disCtx = {0};
    cJSON *BodyData = NULL;
    cJSON *PartitionCursor = NULL;
    cJSON *RecordArray = NULL;
    cJSON *Records = NULL;
    cJSON *RecordData = NULL;
    cJSON *RecordPartitionKey = NULL;
    cJSON *RecordSequenceNumber = NULL;
	cJSON *Timestamp = NULL;
	cJSON *TimestampType = NULL;

    int Len = 0;
    int serrializedMode = DISGetSerializedMode();
    
    DISGetRecords  TempRecord = {0,0,0};
    unsigned long RealLimit = (limit >= 10000) ? 10000 : limit;

    printfdbgfunc;  

    if (g_InitFlag == 0)
    {
        return DISStatusNotInit;
    }

    if((NULL == ResponseInfo) || (NULL == partitionCursor) || (NULL == Endpoint) | (NULL == ProjectId) || (NULL == Region) || (NULL == streamName) || (NULL == NextCursor) || (NULL == handle))
    {
        return DISStatusInvalidParam;
    }
    
    ret = DisValidateDisString(streamName, strlen(streamName), MAX_STREAMNAME_LEN);
    if(DISStatusOK != ret)
    {
        return ret;
    }

    xSecurityToken = (char *)disMemAlloc(MAX_X_SECURITY_TOKEN_LEN);
    if(NULL == xSecurityToken)
    {
        return DISStatusNoMemory;
    }

    ret = g_GetUserAuthInfo(ProjectId, ak, sk, xSecurityToken);
    if(0 != ret)
    {
        disMemFree(xSecurityToken);
        return DISStatusGetUserAuthInfoErr;
    }

    disCtx.ak = ak;
    disCtx.sk = sk;
    disCtx.projectId = ProjectId;
    disCtx.hostname = Endpoint;
    disCtx.region = Region;

    ret = DisGetRequest(&pstTempRequest, DISInterfaceGetRecords, &disCtx, 0);
    if(DISStatusOK != ret)
    {
        disMemFree(xSecurityToken);
        return ret;
    }

    do
    {
        ret = DisComposeUri(pstTempRequest, &disCtx, NULL);
        if(DISStatusOK != ret)
        {
            break;
        }

        ret = DisAddQueryList(pstTempRequest, "partition-cursor", partitionCursor);
        if(DISStatusOK != ret)
        {
            break;
        }

        if(0 != RealLimit)
        {
            //itoa(RealLimit, RecordLimit, 10);
            sprintf(RecordLimit, "%d", (int)RealLimit);
            ret = DisAddQueryList(pstTempRequest, "limit", RecordLimit);
            if(DISStatusOK != ret)
            {
                break;
            }
        }

        ret = DisSendRequest(pstTempRequest, &disCtx, ResponseInfo, xSecurityToken);
        if(DISStatusOK == ret && pstTempRequest->httpResponseCode >=200 && pstTempRequest->httpResponseCode < 300 && BASE64MODE == serrializedMode)
        {
            //logPrint(DISLogLevelInfo,"start process the body: %s\r\n", pstTempRequest->RspBody);
            BodyData = cJSON_Parse(pstTempRequest->RspBody);

            if(NULL != pstTempRequest->jsonData)
            {
                logPrint(DISLogLevelInfo,"parse body error\r\n");
                ret = DISStatusInternalError;
                break;
            }
            pstTempRequest->jsonData = BodyData;

            PartitionCursor = cJSON_GetObjectItem(BodyData, "next_partition_cursor");
            if(NULL == PartitionCursor || NULL == PartitionCursor->valuestring)
            {
                logPrint(DISLogLevelInfo,"the body is not correct: %d, %s\r\n", __LINE__, pstTempRequest->RspBody);
                ret = DISStatusInvalidResponse;
                break;
            }


            Len = strlen(PartitionCursor->valuestring);
            if(Len > 512)
            {
                logPrint(DISLogLevelInfo,"the PartitionCursor too long: %d, %s\r\n", Len, pstTempRequest->RspBody);
                ret = DISStatusInvalidResponse;
                break;
            }

            strncpy((NextCursor->partitionCursor), PartitionCursor->valuestring, Len);
            logPrint(DISLogLevelInfo,"the PartitionCursor is %s\r\n", (char*)(NextCursor->partitionCursor));

            RecordArray = cJSON_GetObjectItem(BodyData, "records");
            if(NULL == RecordArray)
            {
                logPrint(DISLogLevelInfo,"don't have the records in body, %s\r\n", pstTempRequest->RspBody);
                ret = DISStatusInvalidResponse;
                break;
            }

            Base64DecodeBuffer = disMemAlloc(BufferLen);
            if(NULL == Base64DecodeBuffer)
            {
                logPrint(DISLogLevelInfo,"Memory get error\r\n");
                ret = DISStatusNoMemory;
                break;
            }

			int array_size = cJSON_GetArraySize(RecordArray);
			for (Len = 0; Len < array_size; Len++)
            //for(Len = 0; Len < 10000; Len++)
            {
                memset(Base64DecodeBuffer, 0 , BufferLen);
                Records = cJSON_GetArrayItem(RecordArray, Len);
                if(NULL == Records)
                {
                    logPrint(DISLogLevelInfo,"Get %d record error\r\n", Len);
                    break;
                }

                RecordData = cJSON_GetObjectItem(Records, "data");
                RecordPartitionKey  = cJSON_GetObjectItem(Records, "partition_key");
                RecordSequenceNumber = cJSON_GetObjectItem(Records, "sequence_number");
				Timestamp = cJSON_GetObjectItem(Records, "timestamp");
				TimestampType = cJSON_GetObjectItem(Records, "timestamp_type");
                if(NULL == RecordData || NULL == RecordData->valuestring || NULL == RecordPartitionKey || NULL == RecordPartitionKey->valuestring || NULL == RecordSequenceNumber || NULL == RecordSequenceNumber->valuestring || NULL == Timestamp || 0 == Timestamp->valuedouble || NULL == TimestampType || NULL == TimestampType->valuestring)
                {
                    logPrint(DISLogLevelInfo,"the %d record item is incorrect\r\n", Len);
                    ret = DISStatusInvalidResponse;
                    break;
                }


                if(BufferLen < (int)strlen(RecordData->valuestring))
                {
                    logPrint(DISLogLevelInfo,"the buffer len %d is not enough,the data len is %d\r\n", BufferLen, strlen(RecordData->valuestring));
                    BufferLen += strlen(RecordData->valuestring);
                    disMemFree(Base64DecodeBuffer);
                    Base64DecodeBuffer = NULL;

                    Base64DecodeBuffer = (char *)disMemAlloc(BufferLen);
                    if(NULL == Base64DecodeBuffer)
                    {
                        logPrint(DISLogLevelInfo,"Memory get error\r\n");
                        ret = DISStatusNoMemory;
                        break;
                    }
                    memset(Base64DecodeBuffer, 0 , BufferLen);
                }

                TempRecord.data.stringLen = base64_decode((unsigned char *)RecordData->valuestring, (unsigned char *)Base64DecodeBuffer);
                TempRecord.data.data = Base64DecodeBuffer;
                TempRecord.partitionId = RecordPartitionKey->valuestring;
                TempRecord.seqNumber = RecordSequenceNumber->valuestring;
				TempRecord.timestamp = Timestamp->valuedouble;
				TempRecord.timestampType = TimestampType->valuestring;

                handle(streamName, &TempRecord);
            }
        }
		else if (DISStatusOK == ret && pstTempRequest->httpResponseCode >= 200 && pstTempRequest->httpResponseCode < 300 && PROTOBUFFMODE == serrializedMode)
		{
			int loopCount = 0;
			GetRecordsResult *pstGetRecordsResult;
			Record *pstRecords;

			pstGetRecordsResult = get_records_result__unpack(NULL, pstTempRequest->RspBodyCurrentLen, pstTempRequest->RspBody);
			if (NULL == pstGetRecordsResult)
			{
				printf("%s:%d get_records_result__unpack failed\n", __FUNCTION__, __LINE__);
				return DISStatusError;
			}
			else
			{
				for (loopCount = 0; loopCount < pstGetRecordsResult->n_records; loopCount++)
				{
					int dataLen = 0;
					int timeStampLen = 0;
					pstRecords = pstGetRecordsResult->records[loopCount];
					TempRecord.partitionId = pstRecords->partitionkey;
					TempRecord.seqNumber = pstRecords->sequencenumber;
					TempRecord.timestamp = pstRecords->timestamp;
					TempRecord.timestampType = pstRecords->timestamptype;
					if (pstRecords->has_data)
					{
						if (pstRecords->data.data)
						{
							TempRecord.data.stringLen = pstRecords->data.len;
							TempRecord.data.data = pstRecords->data.data;
						}
					}
					
					handle(streamName, &TempRecord);
				}
				get_records_result__free_unpacked(pstGetRecordsResult, NULL);
			}
		}
        else
        {
            logPrint(DISLogLevelError,"When GetRcords DisSendRequest failed .ret = [%d]",ret);
        }
        
    }while(0);

    if(NULL != Base64DecodeBuffer)
    {
        disMemFree(Base64DecodeBuffer);
    }
    disMemFree(xSecurityToken);
    DisReleaseRequest(pstTempRequest);

    return 0;
}