#ifndef _DIS_H_
#define _DIS_H_

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /* __cpluscplus */
#endif /* __cpluscplus */

#include "stdio.h"

#define MAX_STREAMNAME_LEN    64
#define MAX_PARTITIONID_LEN   128 
#define HASH_RANGE_LEN 128
#define SEQUENCE_NUMBER_RANGE_LEN 128
#define DESCRIBE_MAX_PARTITION_NUM 100
#define LIST_MAX_STREAM   100
#define MAX_PARTITION_CURSOR_LEN 512
#define MAX_ENDPOINT_LEN        128
#define MAX_REGION_LEN          32
#define MAX_PROJECTID_LEN       64
#define MAX_AK_LEN  20
#define MAX_SK_LEN  40
#define MAX_X_SECURITY_TOKEN_LEN  (16*1024)

#define BASE_BODY_LEN           1024
#define BASE_URL_LEN            128
#define MAX_QUERY_STRING_LEN    512
#define BASE_RSP_BODY_LEN      2048
#define MAX_RSP_BODY_LEN       (5*1024*1024)
#define MAX_RECORD_LEN         (5 * 1024 * 1024)
#define MAX_RECORD_LEN_COMMON  (1 * 1024 * 1024)
#define MAX_SYNC_RECORD_LEN  (10 * 1024)
#define MAX_RECORD_COUNT   500

#define DISPROPERTIESFILEPATH "./dis.properties"
#define DISVERSIONINFOFILEPATH "./version.properties"
#define DISSERIALIZED_MODE "protobuf"/*protobuf or base64*/

#define MAX_VERSION_INFO_LEN 20
//#define MAX_VERSION_INFO_FILE_LEN 20

//extern char versionPath[MAX_VERSION_INFO_FILE_LEN];
extern char g_Version[MAX_VERSION_INFO_LEN];
/** **************************************************************************
 * Data Types
 ************************************************************************** **/
typedef enum
{
    DISStatusOK,
    DISStatusError,
    DISStatusInternalError,
    DISStatusInvalidStreamName,
    DISStatusInvalidStreamType,
    DISStatusInvalidParam,
    DISStatusNoMemory,
    DISStatusMsgTypeInvalid,
    DISStatusNotInit,
    DISStatusFailedToInitializeRequest,
    DISStatusNameLookupError,
    DISStatusFailToConnect,
    DISStatusOpTimeOut,
    DISStatusCacheOverFlow,
    DISStatusReInit,
    DISStatusGetUserAuthInfoErr,
    DISStatusInvalidResponse,
	DISStatusResponseError,
    DISStatusVersionInitError
}DISStatus;

typedef enum
{
    DISStreamStatusCreating,
    DISStreamStatusRunning,
    DISStreamStatusTerminating,
	DISStreamStatusFrozen,
}DISStreamStatus;

typedef enum
{
	DISDataTypeBlob,
	DISDataTypeJson,
	DISDataTypeCsv,
	DISDataTypeFile,
}DISDataType;

typedef enum
{
    DISStreamTypeCommon,
    DISStreamTypeAdvanced
}DISStreamType;

typedef enum
{
    DISPartitionStatusCreating,
    DISPartitionStatusActive,
    DISPartitionStatusDeleted,
    DISPartitionStatusExpired
}DISPartitionStatus;

typedef enum
{
    DISCursorTypeAtSeqNumber                  = 0, /* AT_SEQUENCE_NUMBER */
    DISCursorTypeAfterSeqNumber               = 1, /* AFTER_SEQUENCE_NUMBER */
    DISCursorTypeTrimHorizon                  = 2, /* TRIM_HORIZON */
    DISCursorTypeLatest                       = 3, /* LATEST */
	DISCursorTypeAtTimestamp				  = 4, /* AT_TIMESTAMP */	
} DISCursorType;

typedef enum
{
    DISQueryParamLimit, 
    DISQueryParamLimitPartitions,
    DISQueryParamStreamName,
    DISQueryParamPartitionId,
    DISQueryParamStartStreamName, 
    DISQueryParamStartPartitionId,
    DISQueryParamPartitionCursor,   
    DISQueryParamCursorType,
    DISQueryParamStartingSequenceNumber,
    DISQueryParamAppName,
    DISQueryParamCheckpointType,
    DISQueryParamTypeAll
} DISQueryParamType;

typedef enum
{
    DISInterfaceCreateStream,
    DISInterfaceDeleteStream,
    DISInterfaceListStream,
    DISInterfaceDescribeStream,
    DISInterfacePutRecords,
    DISInterfaceGetCursor,
    DISInterfaceGetRecords,
    DISInterfaceCreateApp,
    DISInterfaceDeleteApp,
    DISInterfaceCommitCheckpoint,
    DISInterfaceGetCheckpoint,
    DISInterfaceAll,
}DISInterfaceType;

typedef enum
{
    DISHttpMethodGET,
    DISHttpMethodPOST,
    DISHttpMethodDELETE,
    DISHttpMethodUPDATE,
}DISHttpMethod;

typedef enum
{
    DISCheckpointTypeLastRead                  = 0, /* LAST_READ */
} DISCheckpointType;

typedef enum
{
    DISOSWinX86_64,
    DISOSLinuxX86_64
}DISOSType;

typedef struct _DISString
{
    long     stringLen;
    char    *data;
}DISString;

typedef struct _DISResponseInfo
{
    long    HttpResponseCode;
    char    ErrorCode[32];
    char    ErrorDetail[256];
}DISResponseInfo;

typedef struct _DISCreateStream
{
    char           *streamName;
    DISStreamType   streamType;
    long            partitionCount;
	void           *pucReserved;
}DISCreateStream;

typedef struct _ObsDestinationDescriptor
{
	char     *agencyName;
	char     *obsBucketPath;
	void     *pucReserved;
}ObsDestinationDescriptor;

typedef struct _DISCreateStreamExpend
{
	DISDataType     dataType;
	long            dataDuration;
	void           *pucReserved;
	ObsDestinationDescriptor obsDestinationDescriptor;
}DISCreateStreamExpend;

typedef struct _DISDeleteStream
{
    char         *streamName;
}DISDeleteStream;

typedef struct _DISStream
{
    char    streamName[MAX_STREAMNAME_LEN];
}DISStream;

typedef struct _DISListStreamResult
{
    long      totalNumber;
    long      hasMoreStream;
    long      currentNumber;
    DISStream streamList[LIST_MAX_STREAM];
}DISListStreamResult;

typedef struct _DISListStream
{
    long                 limit;
    char                *startStreamName;
    DISListStreamResult  streamListResult;
}DISListStream;

typedef struct _DISPartitionDetailInfo
{
    DISPartitionStatus   partitionStatus;
    char                 partitionId[MAX_PARTITIONID_LEN];
	char				 hashRange[HASH_RANGE_LEN];
	char				 sequenceNumberRange[SEQUENCE_NUMBER_RANGE_LEN];
    //long                 startHashIndex;
    //long                 endHashIndex;
    //long                 startSeqNumber;
    //long                 endSeqNumber;
}DISPartitionDetailInfo;

typedef struct _DISStreamDetailInfo
{
	char                          *streamId;
    char                          *streamName;
    long                           createTime;
    long                           lastModifyTime;
    long                           retentionPeriod;
    DISStreamStatus                streamStatus;
    DISStreamType                  streamType;
    long                           hasMorepartition;
    long                           currentNumber;
	DISDataType					   dataType;
    DISPartitionDetailInfo         partitionList[DESCRIBE_MAX_PARTITION_NUM];
}DISStreamDetailInfo;

typedef struct _DISDescribeStream
{
    char      *streamName;
    long       limitPartitionNum;
    char      *startPartitionId;
    DISStreamDetailInfo  streamInfo;
}DISDescribeStream;

typedef struct  _DISPutRecord
{
    DISString   recordData;
    char       *partitionKey;
    char       *explicitHashKey;
    char       *partitionId;
	void       *pucReserved;
}DISPutRecord;

typedef struct _DISPutRecordsRequestEntryExtendedInfo
{
	char			*fileName;
	char            *deliverDataId;
	long            seqNumber;
	int             endFlag;
}DISPutRecordsRequestEntryExtendedInfo;

typedef struct _DISPutRecordsResult
{
    DISPutRecord    record;
    long            partitionId;
    long            seqNumber;
    long            errorCode;
}DISPutRecordsResult;

typedef struct _DISGetRecords
{
    DISString     data;
    char         *seqNumber;
    char         *partitionId;
	long long      timestamp;
	char           *timestampType;
	void           *pucReserved;
}DISGetRecords;

typedef struct _DISCursor
{
    char                 partitionCursor[MAX_PARTITION_CURSOR_LEN + 8];
}DISCursor;

typedef struct _DISGetCursor
{
    char           *streamName;
    char           *partitionId;
    DISCursorType   cursorType;
    char           *startingSequenceNumber;
    DISCursor       cursorResult;
	char           *timestamp;
	void           *pucReserved;
}DISGetCursor;

typedef DISStatus (*DISGetUserAuthInfp) (char *projectId, char *ak, char *sk, char *xSecurityToken);
typedef DISStatus (*DISGetPropertiesFilePathfp) (char *pucDISPropertyFilePath);

DISStatus DisInit(FILE *logFile, DISGetUserAuthInfp handle);
DISStatus SetUserAuthInfo(char *akArray, char *skArray, char *xSecurityToken);
DISStatus DISSetSerializedMode(char* serializedMode);
void DisDeinit();

DISStatus CreateStream
(
        char             *Endpoint,
        char             *ProjectId,
        char             *Region,
        DISCreateStream  *streamInfo,
        DISResponseInfo  *ResponseInfo
);

DISStatus DeleteStream
(
        char             *Endpoint,
        char             *ProjectId,
        char             *Region,
        char             *streamName,
        DISResponseInfo  *ResponseInfo
);

DISStatus DescribeStream
(
        char              *Endpoint,
        char              *ProjectId,
        char              *Region,
        DISDescribeStream *streamDetail,
        DISResponseInfo   *ResponseInfo
);

DISStatus ListStream
(
        char             *Endpoint,
        char             *ProjectId,
        char             *Region,
        DISListStream    *streamList,
        DISResponseInfo  *ResponseInfo
);

typedef DISStatus (*DISPutRecordCallback)
    (char *errorCode, char *errorDetails, char *streamName, DISPutRecord *putRecord, char *SeqNumber, char *partitionId);

DISStatus PutRecordsAsyn
(
        char                   *Endpoint,
        char                   *ProjectId,
        char                   *Region,
        char                   *streamName,
        DISPutRecord           *putRecord,
        DISPutRecordCallback    handle
);

DISStatus PutRecords
(
        char                   *Endpoint,
        char                   *ProjectId,
        char                   *Region,
        char                   *streamName,
        int                     Count,
        DISPutRecord           *putRecord,
        DISPutRecordCallback    handle,
        DISResponseInfo        *ResponseInfo
);


DISStatus DisGetCursor
(
        char                   *Endpoint,
        char                   *ProjectId,
        char                   *Region,
        DISGetCursor           *getCursor,
        DISResponseInfo        *ResponseInfo
);


typedef DISStatus (*DISGetRecordsCallback)(char *streamName, const DISGetRecords *getRecordsResult);

DISStatus GetRecords
(
        char                  *Endpoint,
        char                  *ProjectId,
        char                  *Region,
        char                  *streamName,
        char                  *partitionCursor,
        unsigned long          limit,
        DISCursor             *NextCursor,
        DISGetRecordsCallback  handle,
        DISResponseInfo       *ResponseInfo
);

#if 0
/** **************************************************************************
 * DIS Callback function
 ************************************************************************** **/

typedef DISStatus (DISPutRecordsCallback)
    (DISStatus status, char *errorDetails, char *streamName, long recordsCount, DISPutRecordsResult  *putRecordsList);

typedef DISStatus (DISGetRecordsCallback)
    (DISStatus status, char *errorDetails, DISString *streamName, const DISGetRecordsResult *getRecordsResult);

/** **************************************************************************
 * DIS basic function
 ************************************************************************** **/





DISStatus PutRecordsSync
(
        DISStreamContext       *disStreamContext,
        long                    recordCount,
        DISPutRecordsResult    *putRecordsList
);

DISStatus PutRecordsAsyn
(
        DISStreamContext       *disStreamContext,
        long                    recordCount,
        DISPutRecord           *putRecordsList,
        DISPutRecordsCallback  *handle
);

DISStatus GetCursor
(
        DISStreamContext *disStreamContext,
        DISGetCursor     *getCursor
);

DISStatus GetRecords
(
        DISStreamContext      *disStreamContext,
        DISString             *partitionCursor,
        long                   limit,
        DISGetRecordsCallback *handler
);

DISStatus CreateApp
(
        DISStreamContext     *disStreamContext,
        char                 *appName
);

DISStatus DeleteApp
(
        DISStreamContext     *disStreamContext,
        char                 *appName
);

DISStatus CommitCheckpoint
(
        DISStreamContext     *disStreamContext,
        char                 *streamName,
        char                 *appName,
        char                 *partitionId,
        DISCheckpointType     checkpointType,
        long                  SeqNumber,
        DISString            *metaData
);


DISStatus GetCheckpoint
(
        DISStreamContext     *disStreamContext,
        DISString            *streamName,
        DISString            *appName,
        DISString            *partitionId,
        DISCheckpointType     checkpointType,
        DISCheckpointResult  *checkPointResult
);

#endif

#ifdef __cplusplus
#if __cplusplus
}
#endif /* __cpluscplus */
#endif /* __cpluscplus */

#endif