#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "log.h"
#include "discommon.h"
#include "dis.h"
#include "disosadapt.h"
#include "disbodyencode.h"
#include "util.h"
#include "httpbody.pb-c.h"
#include "disrequest.h"

int DisBodyEncodeStream(DISCreateStream *streamInfo, char *data, int *bodyLen, int bodyMaxLen)
{
	int    len = 0;
	//char  *pCurrent = NULL;
	//int loop = 0;
	char partitionnum[10] ={0};
	char *streamType = "COMMON";
	
	DISCreateStreamExpend *streamExpend = (DISCreateStreamExpend *)streamInfo->pucReserved;
	char *dataType = "BLOB";
	long dataDuration = 24;

	printfdbgfunc;

	if(NULL == data || NULL == bodyLen || NULL == streamInfo)
	{
		return -1;
	}

	if(DISStreamTypeAdvanced == streamInfo->streamType)
	{
		streamType = "ADVANCED";
	}

	//itoa(streamInfo->partitionCount, partitionnum, 10);
	sprintf(partitionnum,"%d", (int)(streamInfo->partitionCount));

	if (NULL == streamExpend)
	{
		STRING_APPEND(data, len, bodyMaxLen, "{\"stream_name\": \"%s\",\"partition_count\": %d, \"stream_type\": \"%s\"}", streamInfo->streamName, (int)(streamInfo->partitionCount), streamType);
	}
	else
	{
		ObsDestinationDescriptor obsDestinationDescriptor = streamExpend->obsDestinationDescriptor;
		if (NULL != streamExpend->dataDuration && "" != streamExpend->dataDuration)
		{
			dataDuration = streamExpend->dataDuration;
		}

		if (NULL != streamExpend->dataType || "" != streamExpend->dataType)
		{
			dataType = streamExpend->dataType == DISDataTypeBlob ? "BLOB"
				: streamExpend->dataType == DISDataTypeJson ? "JSON"
				: streamExpend->dataType == DISDataTypeCsv ? "CSV"
				: "FILE";
		}

		if (NULL == obsDestinationDescriptor.agencyName || NULL == obsDestinationDescriptor.obsBucketPath)
		{
			STRING_APPEND(data, len, bodyMaxLen, "{\"stream_name\": \"%s\",\"partition_count\": %d, \"stream_type\": \"%s\", \"data_type\": \"%s\", \"data_duration\": \"%ld\"}", streamInfo->streamName, (int)(streamInfo->partitionCount), streamType, dataType, dataDuration);
		}
		else
		{
			STRING_APPEND(data, len, bodyMaxLen, "{\"stream_name\": \"%s\",				\
													\"partition_count\": %d,			\
													\"stream_type\": \"%s\",			\
													\"data_type\": \"%s\",				\
													\"data_duration\": \"%ld\",			\
													\"obs_destination_descriptor\": [{	\
													\"agency_name\": \"%s\"	,			\
													\"obs_bucket_path\": \"%s\"		\
													}] }",
			//STRING_APPEND(data, len, bodyMaxLen, "{\"stream_name\": \"%s\",	\"partition_count\": %d,\"stream_type\": \"%s\",\"data_type\": \"%s\",\"data_duration\": \"%ld\",	\"obs_destination_descriptor\": [{\"agency_name\": \"%s\", \"obs_bucket_path\": \"%s\"}] }",
			streamInfo->streamName, (int)(streamInfo->partitionCount), streamType, dataType, dataDuration, obsDestinationDescriptor.agencyName, obsDestinationDescriptor.obsBucketPath);
		}
	}

	*bodyLen = len;

	printfdbgoutfunc;

	return 0;
}

int DisBodyEncodeAppName(char *appName, char *data, int *bodyLen, int bodyMaxLen)
{
	int    len = 0;
	//char  *pCurrent = NULL;

	printfdbgfunc;

	if(NULL == data || NULL == bodyLen || NULL == appName)
	{
		return -1;
	}

	STRING_APPEND(data, len, bodyMaxLen, "{\"app_name\": \"%s\"}", appName);
	
	*bodyLen = len;

	logPrint(DISLogLevelInfo,"the body is %d:%s\r\n", len, data);

	printfdbgoutfunc;
	return 0;
}

int DisBodyEncodeCheckpoint(char *streamName, char *appName, char *partitionId, char *seqNumber, DISString *metaData, DISCheckpointType type, char *data, int *bodyLen, int bodyMaxLen)
{
	int    len = 0;
	char  *checkpointType = "LAST_READ";
	//type = {0};

	printfdbgfunc;

	if(NULL == data || NULL == bodyLen || NULL == appName || NULL == streamName || NULL == partitionId || NULL == metaData)
	{
		return -1;
	}
	
	STRING_APPEND(data, len, bodyMaxLen, "{\"stream_name\": \"%s\",\"app_name\": \"%s\",\"partition_id\": \"%s\", \"sequence_number\": \"%s\",\"metadata\": \"%s\",\"checkpoint_type\": \"%s\"}", streamName, appName, partitionId, seqNumber, metaData->data,checkpointType);

	*bodyLen = len;

	logPrint(DISLogLevelInfo,"the body is %d:%s\r\n", len, data);

	printfdbgoutfunc;

	return 0;
}

int DisBodyEncodeRecords(char *streamName, DISPutRecord *record, long recordCount, char *data, int *bodyLen, int bodyMaxLen)
{
	int    len = 0, loop = 0;
	char     *pCurrent = NULL;
	DISPutRecord  *tempRecord = NULL;

	printfdbgfunc;

	pCurrent = data;

    /*
        {"stream_name": "stz_test","records":[{"data":"aaaaaaaaaa","partition_key": "1"}]}
    */
	STRING_APPEND(data, len, bodyMaxLen, "{\"stream_name\": \"%s\",\"records\":[", streamName);
	
    int serrializedMode = DISGetSerializedMode();
	for(loop = 0; loop < recordCount; loop++)
	{
		if(0 != loop)
		{
			STRING_APPEND(data, len, bodyMaxLen, "%s", ",");
		}
		tempRecord = &(record[loop]);

		STRING_APPEND(data, len, bodyMaxLen, "%s", "{\"data\":\"");
        
        if(BASE64MODE == serrializedMode)
        {
		    len += base64_encode((unsigned char *)tempRecord->recordData.data, (unsigned char *)&(pCurrent[len]), tempRecord->recordData.stringLen);
        }
        else if(PROTOBUFFMODE == serrializedMode)
        {
            //strncpy((unsigned char *)&(pCurrent[len]),(unsigned char *)tempRecord->recordData.data,tempRecord->recordData.stringLen);
            //len += tempRecord->recordData.stringLen;
            //*bodyLen = protobuf_encode(data, data, len);
        }
		STRING_APPEND(data, len, bodyMaxLen, "\",\"partition_key\": \"%s\"", tempRecord->partitionKey);

		if(NULL != tempRecord->explicitHashKey)
		{
			STRING_APPEND(data, len, bodyMaxLen, ",\"explicit_hash_key\": \"%s\"", tempRecord->explicitHashKey);
		}

		if(NULL != tempRecord->partitionId)
		{
			STRING_APPEND(data, len, bodyMaxLen, ",\"partition_id\": \"%s\"", tempRecord->partitionId);
		}

		DISPutRecordsRequestEntryExtendedInfo *putFileInfo = (DISPutRecordsRequestEntryExtendedInfo*)tempRecord->pucReserved;
		if (NULL != putFileInfo)
		{
			STRING_APPEND(data, len, bodyMaxLen, ",\"extended_info\":{\"file_name\":\"%s\"", putFileInfo->fileName);
			STRING_APPEND(data, len, bodyMaxLen, ",\"deliver_data_id\":\"%s\"", putFileInfo->deliverDataId);
			STRING_APPEND(data, len, bodyMaxLen, ",\"sequence_number\":\"%ld\"", putFileInfo->seqNumber);
			STRING_APPEND(data, len, bodyMaxLen, ",\"end_flag\":\"%s\"", putFileInfo->endFlag == 0 ? "false" : "true");
			STRING_APPEND(data, len, bodyMaxLen, "%s", "}");
		}

		STRING_APPEND(data, len, bodyMaxLen, "%s", "}");
	}
	#if 0
    if(PROTOBUFFMODE == serrializedMode)
    {
	    len += protobuf_encode((unsigned char *)tempRecord->recordData.data, (unsigned char *)&(pCurrent[len]), tempRecord->recordData.stringLen);
    }
    else
    {
        logPrint(DISLogLevelError,"body serialized mode wrong ,mode = %d", serrializedMode);
    }
    #endif
	STRING_APPEND(data, len, bodyMaxLen, "%s", "]}");

    printf("%s:Before-httpbody=%s\n",__FUNCTION__,data);
    if(BASE64MODE == serrializedMode)
    {
        *bodyLen = len;
    }
    else if(PROTOBUFFMODE == serrializedMode)
    {
    }
	printfdbgoutfunc;

	return 0;
}



int DisBodyEncodeRecordsByProtoBuf(char *streamName, DISPutRecord *record, long recordCount, char *data, int *bodyLen, int bodyMaxLen)
{
	int    len = 0, loop = 0;
	char     *pCurrent = NULL;
	DISPutRecord  *tempRecord = NULL;
	DISPutRecordsRequestEntryExtendedInfo *putFileInfo = NULL;
    int inputDataLen = 0 ;
	printfdbgfunc;


    PutRecordsRequestEntryExtendedInfo pucRecordExendInfo = PUT_RECORDS_REQUEST_ENTRY_EXTENDED_INFO__INIT;
    PutRecordsRequestEntry stPutRecordRquestEntry = PUT_RECORDS_REQUEST_ENTRY__INIT;
    PutRecordsRequest stPutRecordRquest = PUT_RECORDS_REQUEST__INIT;

    stPutRecordRquest.streamname = streamName;
    stPutRecordRquest.n_records = recordCount;
    stPutRecordRquest.records = (PutRecordsRequestEntry**)malloc(recordCount * sizeof(PutRecordsRequestEntry*));

    for(loop = 0 ; loop < recordCount ; loop++)
    {
        stPutRecordRquest.records[loop] = malloc(sizeof(stPutRecordRquestEntry));
        put_records_request_entry__init(stPutRecordRquest.records[loop]);
        stPutRecordRquest.records[loop]->has_data = 1;
        stPutRecordRquest.records[loop]->data.len += record[loop].recordData.stringLen;
        stPutRecordRquest.records[loop]->data.data = record[loop].recordData.data;
        //stPutRecordRquest.records[loop]->data = (ProtobufCBinaryData)(record[loop].recordData.data);
        inputDataLen += (record[loop].recordData).stringLen;
        stPutRecordRquest.records[loop]->explicithashkey = record[loop].explicitHashKey;
        stPutRecordRquest.records[loop]->partitionkey = record[loop].partitionKey;
        stPutRecordRquest.records[loop]->partitionid = record[loop].partitionId;


		putFileInfo = (DISPutRecordsRequestEntryExtendedInfo*)record[loop].pucReserved;
		if (NULL != putFileInfo)
		{
			//put_records_request_entry_extended_info__init(stPutRecordRquest.records[loop]->extendedinfo);
			pucRecordExendInfo.filename = putFileInfo->fileName;
			pucRecordExendInfo.deliverdataid = putFileInfo->deliverDataId;
			pucRecordExendInfo.seqnum = putFileInfo->seqNumber;
			pucRecordExendInfo.endflag = putFileInfo->endFlag;
			stPutRecordRquest.records[loop]->extendedinfo = &pucRecordExendInfo;
		}
    }

    len = put_records_request__get_packed_size(&stPutRecordRquest);
    if(len > bodyMaxLen)
    {
        printf("%s:%d packed len exceed the max bufflen;len=%d,maxlen=%d\n",__FUNCTION__,__LINE__,len,bodyMaxLen);
        return DISStatusError;
    }
    
    put_records_request__pack(&stPutRecordRquest,data);

    *bodyLen = len;

    for(loop = 0 ; loop < recordCount ; loop++)
    {
        free(stPutRecordRquest.records[loop]);
    }
    free(stPutRecordRquest.records);
	printfdbgoutfunc;

	return DISStatusOK;
}

int DisBodyDecodeRecordsByProtoBuf(char *pstResponseBody,int bodyLen,DisRequest *pstRequest)
{
    int loopCount = 0;
    char * beginAddr ;
    int itemLen = 0;
    int shardIdLen = 0;
    int sequNumLen = 0;
    int erroCodLen = 0;
    int erroMsgLen = 0;
    PutRecordsResultEntry *pstPutRecordsResultEntry;
    
    if(DISInterfacePutRecords == pstRequest->msgType)
    {
        PutRecordsResult *pstPutRecordsResult;
        pstPutRecordsResult = put_records_result__unpack(NULL,bodyLen,pstResponseBody); 
        if(!pstPutRecordsResult)
        {
			pstRequest->RspBody = pstResponseBody;
            printf("%s:%d put_records_result__unpack failed\n",__FUNCTION__,__LINE__);
            return DISStatusError;
        }
		else
		{
	        beginAddr = pstRequest->RspBody;
	        *(int32_t*)beginAddr = pstPutRecordsResult->failedrecordcount;
	        beginAddr += sizeof(int32_t);
	        *(size_t*)beginAddr = pstPutRecordsResult->n_records;
	        beginAddr += sizeof(size_t);
	        //pstPutRecordsResultC = pstRequest->RspBody;
	        //pstPutRecordsResultC->failedrecordcount = pstPutRecordsResult->failedrecordcount;
	        //pstPutRecordsResultC->n_records = pstPutRecordsResult->n_records;
        
	        for(loopCount = 0 ; loopCount < pstPutRecordsResult->n_records ; loopCount++)
	        {
            
	            pstPutRecordsResultEntry = pstPutRecordsResult->records[loopCount] ;
	            shardIdLen = strlen(pstPutRecordsResultEntry->shardid);
	            sequNumLen = strlen(pstPutRecordsResultEntry->sequencenumber);
            
	            if(pstPutRecordsResultEntry->errorcode)
	            {
	                erroCodLen = strlen(pstPutRecordsResultEntry->errorcode);
	            }
	            else
	            {
	                erroCodLen = 0 ;
	            }
            
	            if(pstPutRecordsResultEntry->errormessage)
	            {
	                erroMsgLen = strlen(pstPutRecordsResultEntry->errormessage); 
	            }
	            else
	            {
	                erroMsgLen = 0 ;
	            }
            
	            itemLen =  shardIdLen + sequNumLen + erroCodLen + erroMsgLen + (erroCodLen + erroMsgLen == 0)?2:((erroCodLen == 0 || erroMsgLen == 0)?3:4);
	            beginAddr += loopCount * itemLen ;
            
	            strncpy(beginAddr , pstPutRecordsResultEntry->shardid , shardIdLen);
	            beginAddr += shardIdLen;
	            *beginAddr = '\0';
	            beginAddr+=1;
	            strncpy(beginAddr , pstPutRecordsResultEntry->sequencenumber , sequNumLen);
	            beginAddr += sequNumLen;
	            *beginAddr = '\0';
	            beginAddr+=1;
            
	            if(!erroCodLen)
	            {
	                strncpy(beginAddr , pstPutRecordsResultEntry->errorcode , erroCodLen);
	                beginAddr += erroCodLen;        
	            }
	            else
	            {
	                ;
	            }
	            *beginAddr = '\0';
	            beginAddr+=1;
            
	            if(!erroMsgLen)
	            {
	                strncpy(beginAddr , pstPutRecordsResultEntry->errormessage , erroMsgLen);
	                beginAddr += erroMsgLen;          
	            }
	            else
	            {
	                ;
	            }
	            *beginAddr = '\0';
	            beginAddr+=1;
	        }
	        put_records_result__free_unpacked(pstPutRecordsResult,NULL);
	    }
	}
    else if(DISInterfaceGetRecords == pstRequest->msgType)
    {
        GetRecordsResult *pstGetRecordsResult;
        Record *pstRecords;
        char *beginAddr ;
        int n_records = 0;
        pstGetRecordsResult = get_records_result__unpack(NULL,bodyLen,pstResponseBody);  
        if(!pstGetRecordsResult)
        {
			pstRequest->RspBody = pstResponseBody;
            printf("%s:%d get_records_result__unpack failed\n",__FUNCTION__,__LINE__);
            return DISStatusError;
        }
		else
		{
	        beginAddr = pstRequest->RspBody;
	        if(pstGetRecordsResult->nextsharditerator)
	        {
	            int nextShardIteratorLen = strlen(pstGetRecordsResult->nextsharditerator);
	            strncpy(beginAddr , pstGetRecordsResult->nextsharditerator , nextShardIteratorLen);
	            printf("%s:%d nextShardIterator = %s , nextShardIteratorLen = %d\n",__FUNCTION__,__LINE__,beginAddr,nextShardIteratorLen);
	            beginAddr += nextShardIteratorLen;
	            *beginAddr = '\0';  
	            beginAddr++;
	        }
	        else
	        {
	            *beginAddr = '\0';
	            beginAddr++;
	        }
        
	        *(size_t*)beginAddr = pstGetRecordsResult->n_records;
	        printf("%s:%d n_records = %d\n",__FUNCTION__,__LINE__,*(size_t*)beginAddr);
	        beginAddr += sizeof(size_t);
        
	        for(loopCount = 0 ; loopCount < pstGetRecordsResult->n_records; loopCount++)
	        {
	            int dataLen = 0;
	            int timeStampLen = 0;
	            pstRecords = pstGetRecordsResult->records[loopCount];
	            shardIdLen = strlen(pstRecords->partitionkey);
	            sequNumLen = strlen(pstRecords->sequencenumber);
	            if(pstRecords->has_data)
	            {
	                if(pstRecords->data.data)
	                {
	                    dataLen = pstRecords->data.len;
	                }
	            }
	            if(pstRecords->has_timestamp)
	            {
	                if(pstRecords->timestamptype)
	                {
	                    timeStampLen = strlen(pstRecords->timestamptype);
	                }
	            }

	            itemLen = shardIdLen + sequNumLen + dataLen + 2*sizeof(int) + sizeof(int64_t) + timeStampLen;
	            beginAddr += loopCount * itemLen;

	            strncpy(beginAddr,pstRecords->partitionkey,shardIdLen);
	            printf("%s:%d partitionkey = %s , shardIdLen = %d\n",__FUNCTION__,__LINE__,beginAddr,shardIdLen);            
	            beginAddr += shardIdLen;
	            *beginAddr = '\0';
	            beginAddr++;

	            strncpy(beginAddr,pstRecords->sequencenumber,sequNumLen);
	            printf("%s:%d sequencenumber = %s , sequNumLen = %d\n",__FUNCTION__,__LINE__,beginAddr,sequNumLen); 
	            beginAddr += sequNumLen;
	            *beginAddr = '\0';
	            beginAddr++;     

            
	            if(pstRecords->has_data)
	            {
	                *(int*)beginAddr = 0xfffffffe;
	                beginAddr += sizeof(int);
	                if(dataLen > 0)
	                {
	                    strncpy(beginAddr,pstRecords->data.data,dataLen);
	                    printf("%s:%d pstRecords->data = %s , dataLen = %d\n",__FUNCTION__,__LINE__,beginAddr,dataLen);
	                    beginAddr += dataLen ;
	                    *beginAddr = '\0';
	                    beginAddr++;
	                }
	                else
	                {
	                    *beginAddr = '\0';
	                    beginAddr++;                    
	                }
	            }
	            else
	            {
	                *(int*)beginAddr = 0xffffffff;
	                beginAddr += sizeof(int);
	            }
            

	            *(int64_t*)beginAddr = pstRecords->timestamp;
	            printf("%s:%d pstRecords->timestamp = %s , timestampLen = %d\n",__FUNCTION__,__LINE__,beginAddr,sizeof(pstRecords->timestamp));
	            beginAddr += sizeof(int64_t);
	            if(pstRecords->has_timestamp)
	            {
	                *(int*)beginAddr = 0xeeeeeeed;
	                if(timeStampLen > 0)
	                {
	                    strncpy(beginAddr,pstRecords->timestamptype,timeStampLen);
	                    printf("%s:%d pstRecords->timestamptype = %s , timeStampLen = %d\n",__FUNCTION__,__LINE__,beginAddr,timeStampLen);
	                    beginAddr += timeStampLen ;
	                    *beginAddr = '\0';
	                    beginAddr++;                    
	                }
	                else
	                {
	                    *beginAddr = '\0';
	                    beginAddr++;                     
	                }
	            }
	            else
	            {
	                *(int*)beginAddr = 0xeeeeeeee;
	            }
	            beginAddr += sizeof(int);
	        }
	        get_records_result__free_unpacked(pstGetRecordsResult,NULL);
	    }
	}
}

#if 0
void DisBodyDecodeRecordsByProtoBuf(char *pucBody,int flag)
{
    PutRecordsResult stPutRecordsResult = PUT_RECORDS_RESULT__INIT;
    PutRecordsResultEntry stPutRecordsResultEntry = PUT_RECORDS_RESULT_ENTRY__INIT;
    GetRecordsRequest stGetRecordsRequest = GET_RECORDS_REQUEST__INIT;
    GetRecordsResult stGetRecordsResult = GET_RECORDS_RESULT__INIT;
    Record stRecord = RECORD__INIT;

    PutRecordsResult *pstPutRecordsResult ;
    int loopCount = 0 ;
    
    if(!pucBody)
    {
        printf("NullInput\n");
    }
    
    int bufferLen = sizeof(pucBody);
    pstPutRecordsResult = put_records_result__unpack(NULL,bufferLen,pucBody);
//DISStatus PutRecordCallBack(char *errorCode, char *errorDetails, char *streamName, DISPutRecord *putRecord, char *SeqNumber, char *partitiodId)
    if(0 == flag)
    {
        for(loopCount = 0 ; loopCount < pstPutRecordsResult->n_records ; loopCount++)
        {
            PutRecordCallBack(0,"test","streamName",0,pstPutRecordsResult->records[loopCount].sequencenumber,pstPutRecordsResult->records[loopCount].shardid);
        }
    }
    
}
#endif
