#include "discommon.h"
#include "dismem.h"
#include "dismemmgt.h"
#include "apr_ring.h"
#include "util.h"
#include "log.h"

int g_InitFlag = 0;
int g_SerializedMode = 0xf0 ;
char g_ak[MAX_AK_LEN] = {0} ;
char g_sk[MAX_SK_LEN] = {0} ;
char g_endpoint[MAX_ENDPOINT_LEN] = {0} ;
char g_projectid[MAX_PROJECTID_LEN] = {0} ;
char g_region[MAX_REGION_LEN] = {0} ;
unsigned long cryptTable[0x500] = {0};
SendInfoHashTable    g_stSendInfoTable = {0};
DISGetUserAuthInfp   g_GetUserAuthInfo = NULL;
pthread_mutex_t mutex;

DISStatus GetUserConfInfo(char *pucConfigFilePath)
{
    FILE *pfTmp = 0 ;

    printfdbgfunc;
    
    if(!pucConfigFilePath || !strlen(pucConfigFilePath))
    {
        printf("\r\n InputPara Error") ;
        return DISStatusError;
    }
    
    if(0 == access(pucConfigFilePath,R_OK))
    {
        ;
    }
    else
    {
        printf("\r\nInput Config File Does Not Exist") ;
        return DISStatusError ;
    }

    pfTmp = fopen(pucConfigFilePath,"r") ;
    if(!pfTmp)
    {
        return DISStatusError ;
    }
    
    char *pucBuffer = disMemAlloc(MAX_LINE_LEN + 1);
    if(!pucBuffer)
    {
        printf("malloc mem failed\n");
        return DISStatusError;
    }
    memset(pucBuffer,0,MAX_LINE_LEN + 1);
    
    while(fgets(pucBuffer , MAX_LINE_LEN , pfTmp))
    {
        if(!strstr(pucBuffer,"endpoint="))
        {
            strncpy(g_endpoint,pucBuffer + strlen("endpoint="),MAX_ENDPOINT_LEN) ;
        }

        if(!strstr(pucBuffer,"region="))
        {
            strncpy(g_region,pucBuffer + strlen("region="),MAX_REGION_LEN) ;
        }

        if(!strstr(pucBuffer,"ak="))
        {
            strncpy(g_ak,pucBuffer + strlen("ak="),MAX_AK_LEN) ;
        }

        if(!strstr(pucBuffer,"sk="))
        {
            strncpy(g_sk,pucBuffer + strlen("sk="),MAX_SK_LEN) ;
        }    
        
        if(!strstr(pucBuffer,"projectId="))
        {
            strncpy(g_projectid,pucBuffer + strlen("projectId="),MAX_PROJECTID_LEN) ;
        }     
        memset(pucBuffer,0,MAX_LINE_LEN+1);
    }
    
    disMemFree(pucBuffer);
    fclose(pfTmp);
    
    if(g_InitFlag == 1)
    {
        g_InitFlag = 2;
    }
    return DISStatusOK;
}

DISStatus GetUserAuthInfoFromPropertyFile(char *ProjectId, char *akArray, char *skArray, char *xSecurityToken)
{
    if(g_InitFlag == 2)
    {
        if(DISStatusOK == GetUserConfInfo(DISPROPERTIESFILEPATH))
        {
            strncpy(ProjectId,g_projectid,strlen(g_projectid));
            strncpy(akArray,g_ak,strlen(g_ak));
            strncpy(skArray,g_sk,strlen(g_sk));
        }
        else
        {
            return DISStatusError;
        }
    }
    else
    {
        strncpy(ProjectId,g_projectid,strlen(g_projectid));
        strncpy(akArray,g_ak,strlen(g_ak));
        strncpy(skArray,g_sk,strlen(g_sk));
    }
    return DISStatusOK;
}

int DISGetSerializedMode()
{

    if(g_SerializedMode & 0x0001)
    {
        return BASE64MODE;
    }
    else if(g_SerializedMode & 0x0002)
    {
        return PROTOBUFFMODE;
    }
    else
    {
        if(!strcmp(DISSERIALIZED_MODE,"base64"))
        {
            g_SerializedMode = 1 ;
            return BASE64MODE ;
        }
        else if(!strcmp(DISSERIALIZED_MODE,"protobuf"))
        {
            g_SerializedMode = 2 ;
            return PROTOBUFFMODE ;
        }
        else
        {
            return DISStatusInternalError ;
        }
    }
}

int GetBase64Mode()
{
	return BASE64MODE;
}

int GetProtobufMode()
{
	return PROTOBUFFMODE;
}

DISStatus SetUserAuthInfo(char *akArray, char *skArray, char *xSecurityToken)
{

    printfdbgfunc;
    if(!(akArray && skArray && strlen(akArray) && strlen(skArray)))
    {
        printf("Input Para Error\n") ;
        return DISStatusError;
    }
    if(strlen(akArray) > MAX_AK_LEN || strlen(skArray) > MAX_SK_LEN)
    {
        printf("Input Para Len Erro\n");
        return DISStatusError;
    }
    strncpy(g_ak,akArray,MAX_AK_LEN);
    strncpy(g_sk,skArray,MAX_SK_LEN);
    
    printfdbgoutfunc;

    return DISStatusOK;
}

int DisRecordListInit(int MaxNum, DisRecordList **list)
{
	DisRecordList *pTemp = NULL;
	int ret = 0;
	int num = (MaxNum <= 1024 ? 1024 : MaxNum);

	printfdbgfunc;

	if(NULL == list)
	{
		return (int)DISStatusInvalidParam;
	}

	pTemp = (DisRecordList *)disMemAlloc(sizeof(DisRecordList));
	if(NULL == pTemp)
	{
		return (int)DISStatusNoMemory;
	}

	pTemp->CurrentCount = 0;
	pTemp->MaxCount = num;

	ret = SmCreate(&pTemp->mutex);
	if(0 != ret)
	{
		disMemFree(pTemp);
		return ret;
	}

	APR_RING_INIT(&(pTemp->list),  _DisRecordListItem, RecordListLink);
	pTemp->InitFlag = 1;
	pTemp->AllInputCount = 0;
	pTemp->AllOutputCount = 0;

	*list = pTemp;

	printfdbgoutfunc;
	
	return 0;
}

int DisRecordListDeinit(DisRecordList *list)
{

	printfdbgfunc;

	if(NULL == list)
	{
		return 0;
	}

	if(0 != list->CurrentCount || 0 == list->InitFlag)
	{
		return (int)DISStatusInternalError;
	}

	SmDelete(&(list->mutex));
	disMemFree(list);

	printfdbgoutfunc;
	return 0;
}

DisRecordListItem* DisRecordListItemAlloc(DISPutRecord *data)
{
	int len = 0;
	DisRecordListItem *pTemp = NULL;

	printfdbgfunc;	

	if(NULL == data || NULL == data->partitionKey)
	{
		return NULL;
	}

	len = sizeof(struct _DisRecordListItem) + data->recordData.stringLen + 1;

	pTemp = (DisRecordListItem *)disMemAlloc(len);
	if(NULL == pTemp)
	{
		return NULL;
	}

	/* 将输入的信息复制到申请的item中 */
	strncpy(pTemp->partitionKey, data->partitionKey, strlen(data->partitionKey));

	if(NULL != data->partitionId && strlen(data->partitionId) <= 64)
	{
		strncpy(pTemp->partitionId, data->partitionId, strlen(data->partitionId));
	}

	if(NULL != data->explicitHashKey && strlen(data->explicitHashKey) <= 16)
	{
		strncpy(pTemp->explicitHashKey, data->explicitHashKey, strlen(data->explicitHashKey));
	}

	if(0 != data->recordData.stringLen && NULL != data->recordData.data)
	{
		memcpy(pTemp->data, data->recordData.data, data->recordData.stringLen);
	}
	pTemp->RawData.stringLen = data->recordData.stringLen;
	pTemp->RawData.data = pTemp->data;

	APR_RING_ELEM_INIT(pTemp, RecordListLink);

	printfdbgoutfunc;

	return pTemp;
}

void DisRecordListItemRelease(DisRecordListItem *node)
{
	printfdbgfunc;
	if(NULL == node)
	{
		return ;
	}

	disMemFree(node);
	printfdbgoutfunc;
	return;
}

int DisRecordListInsertToHead(DisRecordList *list, DisRecordListItem *node)
{

	printfdbgfunc;
	if(NULL == list || NULL == node || 1 != list->InitFlag)
	{
		return (int)DISStatusInvalidParam;
	}

	SmP(&(list->mutex), 0);

	APR_RING_INSERT_HEAD(&(list->list), node, _DisRecordListItem, RecordListLink);
	list->CurrentCount++;
	list->AllInputCount++;

	SmV(&(list->mutex));

	printfdbgoutfunc;

	return 0;
}

int DisRecordListInsertToTail(DisRecordList *list, DisRecordListItem *node)
{
	int ret = 0;

	printfdbgfunc;

	if(NULL == list || NULL == node || 1 != list->InitFlag)
	{
		return (int)DISStatusInvalidParam;
	}

	SmP(&(list->mutex), 0);

	do
	{
		if(list->CurrentCount >= list->MaxCount)
		{
			ret = (int)DISStatusCacheOverFlow;
			break;
		}

		APR_RING_INSERT_TAIL(&(list->list), node, _DisRecordListItem, RecordListLink);
		list->CurrentCount++;
		list->AllInputCount++;
	}while(0);

	SmV(&(list->mutex));

	printfdbgoutfunc;

	return ret;
}

DisRecordListItem* DisRecordListGetNode(DisRecordList *list)
{

	int ret = 0;
	DisRecordListItem *node = NULL;

	printfdbgfunc;

	if(NULL == list || 1 != list->InitFlag)
	{
		return NULL;
	}

	SmP(&(list->mutex), 0);

	do
	{
		if(0 == list->CurrentCount)
		{
			node = NULL;
			break;
		}

		node = APR_RING_FIRST(&(list->list));
		if(NULL != node)
		{
			APR_RING_REMOVE(node, RecordListLink);
			list->CurrentCount--;
			list->AllOutputCount++;
		}
	}while(0);

	SmV(&(list->mutex));

	printfdbgoutfunc;

	return node;
}

void DisListDebug(DisRecordList *list)
{
	DisRecordListItem  *pCur;
	DisRecordListItem  *pNext;
	int count = 0;

	printfdbgfunc;

	APR_RING_FOREACH_SAFE(pCur, pNext, &(list->list), _DisRecordListItem, RecordListLink)
	{
		//printf("the %d record: partitionkey is %s, data len %d, data is %s\r\n", count, pCur->partitionKey, pCur->RawData.stringLen, pCur->RawData.data);
		count++;
	}

	printf("The list count is %d, input all is %d, output all count is %d\r\n", list->CurrentCount, list->AllInputCount, list->AllOutputCount);

	if(count != list->CurrentCount)
	{
		printf("Fator Error\r\n");
	}

	printfdbgoutfunc;

}



void prepareCryptTable()
{ 
    unsigned long seed = 0x00100001, index1 = 0, index2 = 0, i;
 
    for( index1 = 0; index1 < 0x100; index1++ )
    { 
        for( index2 = index1, i = 0; i < 5; i++, index2 += 0x100 )
        { 
            unsigned long temp1, temp2;
 
            seed = (seed * 125 + 3) % 0x2AAAAB;
            temp1 = (seed & 0xFFFF) << 0x10;
 
            seed = (seed * 125 + 3) % 0x2AAAAB;
            temp2 = (seed & 0xFFFF);
 
            cryptTable[index2] = ( temp1 | temp2 ); 
       } 
   } 
} 

int DisSendInfoHashTableInit(int MemCareFlag)
{
	int count  = (MemCareFlag == 0 ? 5471 : 17);
	SendInfoHashItem *pTemp = NULL;
	int ret = 0;

	printfdbgfunc;

	if(0 != g_stSendInfoTable.InitFlag)
	{
		return (int)DISStatusReInit;
	}

	pTemp = (SendInfoHashItem *)disMemAlloc(sizeof(SendInfoHashItem) * count);
	if(NULL == pTemp)
	{
		return (int)DISStatusNoMemory;
	}
	memset(pTemp, 0, (sizeof(SendInfoHashItem) * count));
	prepareCryptTable();

	ret = SmCreate(&(g_stSendInfoTable.mutex));
	if(0 != ret)
	{
		disMemFree(pTemp);
		return ret;
	}
	g_stSendInfoTable.SendInfoItemList = pTemp;
	g_stSendInfoTable.CurrentCount = 0;
	g_stSendInfoTable.InitFlag = 1;
	g_stSendInfoTable.MaxCount = count;

	printfdbgoutfunc;

	return 0;
}

void DisSendInfoHashTableDeInit()
{
	return;
}

unsigned long HashString( char *lpszFileName, unsigned long dwHashType )
{ 
    unsigned char *key  = (unsigned char *)lpszFileName;
    unsigned long seed1 = 0x7FED7FED;
    unsigned long seed2 = 0xEEEEEEEE;
    int ch;
 
    while( *key != 0 )
    { 
        ch = toupper(*key++);
 
        seed1 = cryptTable[(dwHashType << 8) + ch] ^ (seed1 + seed2);
        seed2 = ch + seed1 + seed2 + (seed2 << 5) + 3; 
    }
    return seed1; 
}

int GetHashTablePos( char *RawString, SendInfoHashTable *lpTable)
{
    const int  HASH_OFFSET = 0, HASH_A = 1, HASH_B = 2;
	long TableSize = lpTable->MaxCount;
	SendInfoHashItem* ItemArray = lpTable->SendInfoItemList;

    unsigned long  nHash = HashString( RawString, HASH_OFFSET );
    unsigned long  nHashA = HashString( RawString, HASH_A );
    unsigned long  nHashB = HashString( RawString, HASH_B );
    unsigned long  nHashStart = nHash % TableSize;
    unsigned long  nHashPos = nHashStart;
 
	printfdbgfunc;
	printf("the pos is %d, the max is %d\r\n", nHashPos, TableSize);

    while ( ItemArray[nHashPos].bExists )
   {
        if ( ItemArray[nHashPos].nHashA == nHashA
        &&  ItemArray[nHashPos].nHashB == nHashB )
       {
		    printfdbgoutfunc;
            return nHashPos;
       }
       else
       {
            nHashPos = (nHashPos + 1) % TableSize;
       }
 
        if (nHashPos == nHashStart)
              break;
    }
	printfdbgoutfunc;
    return -1;
}

int HashInsert(char *RawString, SendInfoHashTable *lpTable)
{ 
	const unsigned long HASH_OFFSET = 0, HASH_A = 1, HASH_B = 2; 

	unsigned long TableSize = lpTable->MaxCount;
	unsigned long nHash = HashString(RawString, HASH_OFFSET); 
	unsigned long nHashA = HashString(RawString, HASH_A); 
	unsigned long nHashB = HashString(RawString, HASH_B); 
	unsigned long nHashStart = nHash % TableSize; 
	SendInfoHashItem* ItemArray = lpTable->SendInfoItemList;
	int nHashPos = nHashStart;

	printfdbgfunc;

	while( ItemArray[nHashPos].bExists) 
	{ 
		nHashPos = (nHashPos + 1) % TableSize; 
		if (nHashPos == nHashStart) 
		{ 
			return -1; 
		} 
	} 
	ItemArray[nHashPos].bExists = 1; 
	ItemArray[nHashPos].nHashA = nHashA; 
	ItemArray[nHashPos].nHashB = nHashB;
	lpTable->CurrentCount++;

	printfdbgoutfunc;

	return nHashPos; 
}

void HashDelete(int pos, SendInfoHashTable *lpTable)
{ 
	unsigned long TableSize = lpTable->MaxCount;
	SendInfoHashItem* ItemArray = lpTable->SendInfoItemList;

	printfdbgfunc;

	if(pos >= TableSize)
	{
		return;
	}

	if( ItemArray[pos].bExists != 1) 
	{ 
		return ; 
	} 
	ItemArray[pos].bExists = 0; 
	ItemArray[pos].nHashA = 0; 
	ItemArray[pos].nHashB = 0;
	lpTable->CurrentCount--;

	printfdbgoutfunc;

	return; 
}

int DisInsertToSendInfoList(char *Endpoint, char *ProjectId, char *Region, char *StreamName)
{
	char  HashKey[512] = {0};
	int len = 0;
	int index = -1;
	SendInfoHashTable *pHashTable = &g_stSendInfoTable;
	SendInfo  *pTempSendInfo = NULL;
	int ret = 0;

	printfdbgfunc;

	STRING_APPEND(HashKey, len, 512, "%s%s%s%s", ProjectId, Endpoint, Region, StreamName);

	SmP(&(pHashTable->mutex), 0);
	do
	{
		index = GetHashTablePos(HashKey, pHashTable);
		if(-1 != index)
		{
			break;
		}

		index = HashInsert(HashKey, pHashTable);
		if(-1 == index)
		{
			break;
		}

		pTempSendInfo = &(pHashTable->SendInfoItemList[index].SendInfoDetail);
		strncpy(pTempSendInfo->Endpoint, Endpoint, strlen(Endpoint));
		strncpy(pTempSendInfo->ProjectId, ProjectId, strlen(ProjectId));
		strncpy(pTempSendInfo->Region, Region, strlen(Region));
		strncpy(pTempSendInfo->StreamName, StreamName, strlen(StreamName));

		ret = DisRecordListInit(4096, &(pHashTable->SendInfoItemList[index].RecordList));
		if(0 != ret)
		{
			HashDelete(index, pHashTable);
			memset(pTempSendInfo, 0, sizeof(SendInfo));
			index = -1;
		}
	}while(0);
	SmV(&(pHashTable->mutex));

	printfdbgoutfunc;

	return index;
}


int DisGetSendRecordList(char *Endpoint, char *ProjectId, char *Region, char *StreamName, DisRecordList **RecordList)
{
	char  HashKey[512] = {0};
	int len = 0;
	int index = -1;
	SendInfoHashTable *pHashTable = &g_stSendInfoTable;
	DisRecordList *TempList = NULL;

	printfdbgfunc;

	STRING_APPEND(HashKey, len, 512, "%s%s%s%s", ProjectId, Endpoint, Region, StreamName);

	index = GetHashTablePos(HashKey, pHashTable);
	if(-1 == index)
	{
		index = DisInsertToSendInfoList(Endpoint, ProjectId, Region, StreamName);
	}
	
	if(-1 != index)
	{
		TempList = pHashTable->SendInfoItemList[index].RecordList;
	}

	*RecordList = TempList;	

	printfdbgoutfunc;

	return 0;
}

int DisCheckPutRecord(DISPutRecord *putRecord)
{
	printfdbgfunc;

	if(NULL == putRecord || NULL == putRecord->partitionKey)
	{
		return DISStatusInvalidParam;
	}

	if (putRecord->recordData.stringLen > MAX_SYNC_RECORD_LEN)
	{
		return DISStatusInvalidParam;
	}

	if(0 == putRecord->recordData.stringLen && NULL != putRecord->recordData.data)
	{
		return DISStatusInvalidParam;
	}

	printfdbgoutfunc;

	return 0;
}

void DisHashTableDbgInfo()
{
	SendInfoHashTable *pHashTable = &g_stSendInfoTable;
	int loop = 0;
	SendInfoHashItem *item = NULL;
	int count = 0;


	printf("the current count is %d\r\n", pHashTable->CurrentCount);
	for(loop = 0; loop < pHashTable->MaxCount; loop++)
	{
		item = &(pHashTable->SendInfoItemList[loop]);
		if(item->bExists)
		{
			count++;
			printf("the Endpoint: %s, ProjectId: %s, StreamName: %s\r\n", item->SendInfoDetail.Endpoint, item->SendInfoDetail.ProjectId, item->SendInfoDetail.StreamName);
			DisListDebug(item->RecordList);
		}
	}	

	if(count != pHashTable->CurrentCount)
	{
		printf("[Fator Error!!!], the count %d is not match %d\r\n", count, pHashTable->CurrentCount);
	}
}