
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "dismem.h"
#include "log.h"

/* add for memory buffer */

#define  MEM_TAIL        0x87654321
#define  MEM_TAIL_LEN    sizeof(unsigned long)
#define  MEM_HEAD        0x12345678
#define  SEG_SIZE  32   /* memory cb segregate */
#define  INVALID_MEM_INDEX 0xFFFFFFFF

#define CORE_GET_MEM_TAIL(_mem_ptr, _blk_size) (unsigned int*)((unsigned int)_mem_ptr + _blk_size)
#define CORE_SET_MEM_TAIL(_mem_ptr, _blk_size) ((*(unsigned int*)((unsigned int)_mem_ptr + _blk_size)) = MEM_TAIL)

#define CORE_GET_MEM(_mem_ptr, _index, _blk_size) \
    (void *)((unsigned int)_mem_ptr + (_blk_size + MEM_TAIL_LEN) * _index)


unsigned int mem_region_init(MEM_REGION *regionInfo, MEM_CFG *MemCfg, unsigned int count,unsigned int realloc_flag,unsigned int check_flag)
{
    MEM_CFG      *mem_cfg_p = MemCfg;
    unsigned int  type_count = count;
    MEM_CB       *mem_cb_p;
    unsigned int  size, i, j, type_index;
    MEM_REGION   *mem_region_p = NULL;


    unsigned int     ret_val = 0;

	printfdbgfunc;

    if(NULL == regionInfo)
    {
        return 1;
    }

    mem_region_p = regionInfo;

    mem_region_p->realloc_flag = realloc_flag;
    mem_region_p->check_flag = check_flag;

    if ((NULL == MemCfg) || (0 == count))
    {
        return 1;
    }

    mem_region_p->type_count = type_count;
    size = sizeof(MEM_ELMNT) * type_count + SEG_SIZE;
    mem_region_p->mem_elmnt_info_p = malloc(size);
    if (NULL == mem_region_p->mem_elmnt_info_p)
    {
        return 1;
    }


    (void)memset(mem_region_p->mem_elmnt_info_p, 0, size);

    for (i = 0; i < type_count; i++)
    {
        if ((0 == mem_cfg_p[i].blk_size) || (0 == mem_cfg_p[i].max_blk_num))
        {
            mem_region_deinit(regionInfo);
            return 1;
        }
            
        mem_region_p->mem_elmnt_info_p[i].blk_size = mem_cfg_p[i].blk_size;
        mem_region_p->mem_elmnt_info_p[i].max_blk_num = mem_cfg_p[i].max_blk_num;
        mem_region_p->mem_elmnt_info_p[i].max_used_num = 0;
        mem_region_p->mem_elmnt_info_p[i].free_blk_num = mem_cfg_p[i].max_blk_num;

        size = sizeof(MEM_CB) * mem_cfg_p[i].max_blk_num + SEG_SIZE;
        mem_region_p->mem_elmnt_info_p[i].mem_cb_p = (MEM_CB *)malloc(size);
        if (NULL == mem_region_p->mem_elmnt_info_p[i].mem_cb_p)
        {
            mem_region_deinit(regionInfo);
            return 1;
        }
        memset(mem_region_p->mem_elmnt_info_p[i].mem_cb_p, 0, size);
    }

    for (i = 0; i < type_count; i ++)
    {
        size = (mem_cfg_p[i].blk_size + MEM_TAIL_LEN) * (mem_cfg_p[i].max_blk_num + 1);
        mem_region_p->mem_elmnt_info_p[i].mem_s_ptr = malloc(size);
        if (NULL == mem_region_p->mem_elmnt_info_p[i].mem_s_ptr)
        {
            mem_region_deinit(regionInfo);
            return 1;
        }
        memset(mem_region_p->mem_elmnt_info_p[i].mem_s_ptr, 0, size);
        
        mem_region_p->mem_elmnt_info_p[i].mem_e_ptr = (void*)((unsigned int)mem_region_p->mem_elmnt_info_p[i].mem_s_ptr + size);

        mem_cb_p = mem_region_p->mem_elmnt_info_p[i].mem_cb_p;
        for (j = 0; j < mem_cfg_p[i].max_blk_num; j++)
        {
            mem_cb_p[j].next_index = (j + 1);
            mem_cb_p[j].mem_ptr = CORE_GET_MEM(mem_region_p->mem_elmnt_info_p[i].mem_s_ptr, j, mem_cfg_p[i].blk_size);
            mem_cb_p[j].alloc_tick = 0;
        }

        mem_cb_p[j - 1].next_index = INVALID_MEM_INDEX;
    }


    mem_region_p->mem_use_info.block_type = type_count;


    mem_region_p->mem_use_info.block_info = malloc(type_count * sizeof(BLK_INFO));

    if (NULL == mem_region_p->mem_use_info.block_info)
    {
        mem_region_deinit(mem_region_p);
        return 1;
    }
    
    memset(mem_region_p->mem_use_info.block_info, 0, type_count * sizeof(BLK_INFO));

    for (type_index = 0; type_index < type_count; type_index++)
    {
       mem_region_p->mem_use_info.block_info[type_index].block_size   = mem_region_p->mem_elmnt_info_p[type_index].blk_size;
       mem_region_p->mem_use_info.block_info[type_index].max_blk_num  = mem_region_p->mem_elmnt_info_p[type_index].max_blk_num;
       mem_region_p->mem_use_info.block_info[type_index].free_blk_num = mem_region_p->mem_elmnt_info_p[type_index].free_blk_num;
    }   

	printfdbgoutfunc;
    
    return ret_val;
}




void mem_region_deinit(MEM_REGION *regionInfo)
{
    MEM_REGION   *mem_region_p = NULL;
    MEM_USE_INFO *mem_use_info_p = NULL;
    unsigned int  count = 0;

	printfdbgfunc;

    if (NULL == regionInfo)
    {
        return ;
    }

    mem_region_p = regionInfo;
    mem_use_info_p = &(mem_region_p->mem_use_info);

    if (mem_region_p->mem_elmnt_info_p != NULL)
    {
        for (count = 0; count < mem_region_p->type_count; count++)
        {
            if (mem_region_p->mem_elmnt_info_p[count].mem_cb_p != NULL)
            {
                free(mem_region_p->mem_elmnt_info_p[count].mem_cb_p);
                mem_region_p->mem_elmnt_info_p[count].mem_cb_p = NULL;
            }
            
            if (mem_region_p->mem_elmnt_info_p[count].mem_s_ptr != NULL)
            {
                free(mem_region_p->mem_elmnt_info_p[count].mem_s_ptr);
                mem_region_p->mem_elmnt_info_p[count].mem_s_ptr = NULL;
                mem_region_p->mem_elmnt_info_p[count].mem_e_ptr = NULL;
            }
        }

        free(mem_region_p->mem_elmnt_info_p);

    }

    if (mem_use_info_p->block_info != NULL)
    {
        free(mem_use_info_p->block_info);
        mem_use_info_p->block_type = 0;
        mem_use_info_p->block_info = NULL;
    }

    memset(mem_region_p, 0, sizeof(MEM_REGION));

	printfdbgoutfunc;

}



void *core_malloc(MEM_REGION *regionInfo, unsigned int size ,unsigned int *OUT_size)
{
    MEM_CB       *mem_cb_p;
    void         *mem_ptr = NULL;
    unsigned int  type, index;
    unsigned int *real_size_p = OUT_size;
    unsigned int  total_size = 0;
    unsigned int max_used_blk;
    unsigned int mem_blk_type;
    MEM_ELMNT   *mem_elmnt_p;
    MEM_REGION  *mem_region_p;
    unsigned int ext_size = 0;

	printfdbgfunc;

    mem_region_p = regionInfo;
    mem_elmnt_p  = mem_region_p->mem_elmnt_info_p;
    mem_blk_type = mem_region_p->type_count;
    

    size = size;

    for (type = 0; type < mem_blk_type; type++)
    {
        if (mem_elmnt_p[type].blk_size >= size)
        {
            if (0 == mem_elmnt_p[type].free_blk_num)
            {
                ext_size = mem_elmnt_p[type].blk_size;
            }
            else
            {
                break;
            }
        } 
        else
        {
            ext_size = size;
        }
    }

    if (type == mem_blk_type)
    {
        if (mem_region_p->realloc_flag)
        {
            if (real_size_p != NULL)
            {
                ext_size = CORE_ALIGN(size) + MEM_TAIL_LEN;
                *real_size_p = ext_size;
            }

            mem_ptr = malloc(ext_size);
        }
        else
        {
            return NULL;
        }
    }
    else
    {
        if (real_size_p != NULL)
        {
            *real_size_p = mem_elmnt_p[type].blk_size;
        }

        index = mem_elmnt_p[type].free_cb_index;
        mem_cb_p = &(mem_elmnt_p[type].mem_cb_p[index]);
        mem_cb_p->alloc_tick = 1;
        
        mem_ptr = mem_cb_p->mem_ptr;


        mem_elmnt_p[type].free_cb_index = mem_cb_p->next_index;
        mem_elmnt_p[type].free_blk_num --;


        max_used_blk = mem_elmnt_p[type].max_blk_num - mem_elmnt_p[type].free_blk_num;
        if (mem_elmnt_p[type].max_used_num < max_used_blk)
        {
            mem_elmnt_p[type].max_used_num = max_used_blk;
        }
        
        CORE_SET_MEM_TAIL(mem_ptr, mem_elmnt_p[type].blk_size);
    }

	printfdbgoutfunc;
        
    return mem_ptr;
}

unsigned int core_free(MEM_REGION *regionInfo,  void * mem_free )
{
    unsigned int   index, type;
    unsigned int   tail;
    void          *mem_ptr;
    unsigned int   mem_blk_type;
    MEM_ELMNT     *mem_elmnt_p;
    MEM_REGION    *mem_region_p;

	printfdbgfunc;

    mem_region_p = regionInfo;
    mem_elmnt_p  = mem_region_p->mem_elmnt_info_p;
    mem_blk_type = mem_region_p->type_count;
    

    mem_ptr = mem_free;
    
    if (NULL == mem_ptr)
    {
        return 0;
    }

    if (NULL == mem_elmnt_p)
    {
        if (mem_region_p->realloc_flag)
        {
            free(mem_ptr);
        }
        return 0;
    }
    
    for (type = 0; type < mem_blk_type; type++)
    {
        if ((mem_ptr >= mem_elmnt_p[type].mem_s_ptr) &&
            (mem_ptr <= mem_elmnt_p[type].mem_e_ptr))
        {
            break;
        }
    }

    if (type >= mem_blk_type)
    {
        if (mem_region_p->realloc_flag)
        {
            free(mem_ptr);
            return 0;
        }
        
        return 1;
    }

    index = (unsigned int)((unsigned int)mem_ptr - (unsigned int)mem_elmnt_p[type].mem_s_ptr)/(mem_elmnt_p[type].blk_size + MEM_TAIL_LEN);

    /* already be freed */
    if(mem_elmnt_p[type].mem_cb_p[index].alloc_tick == 0)
    {
        /* invalid mem cb */
        return 2;
    }
    
    /* insert current mem_cb into free mem_cb */
    mem_elmnt_p[type].mem_cb_p[index].next_index = mem_elmnt_p[type].free_cb_index;

    mem_elmnt_p[type].mem_cb_p[index].alloc_tick = 0;
    mem_elmnt_p[type].free_cb_index = index;
    mem_elmnt_p[type].free_blk_num ++;

    tail = *(unsigned int*)CORE_GET_MEM_TAIL(mem_ptr, mem_elmnt_p[type].blk_size);

    if(MEM_TAIL != tail)
    {
        /* invalid mem cb */
        return 3;       
    }

	printfdbgoutfunc;

    return 0;
}



