#ifndef _MEM_H_
#define _MEM_H_

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /* __cpluscplus */
#endif /* __cpluscplus */

#define COMMON_MAGIC_TAIL    0x12345678

#define CORE_ALIGN(_size) ((_size + 7) & (~7))

typedef struct MEM_CFG
{
    unsigned int blk_size;             /* memory block size */
    unsigned int max_blk_num;          /* max block number  */    
}MEM_CFG;

typedef struct MEM_CB
{
    unsigned int next_index;           /* next memory block index */
    unsigned int alloc_tick;           /* allocate ticks */
    void        *mem_ptr;              /* memory block header */
}MEM_CB;

typedef struct _MEM_ELMNT
{
    unsigned int blk_size;             /* mem block size */
    unsigned int max_blk_num;          /* mem block number */
    unsigned int max_used_num;         /* mem used block size */
    unsigned int free_blk_num;         /* free mem block number */
    unsigned int free_cb_index;        /* current free mem block index */
    MEM_CB      *mem_cb_p;             /* mem control block point */
    void        *mem_s_ptr;            /* mem start point */
    void        *mem_e_ptr;            /* mem end point */
}MEM_ELMNT;

typedef struct BLK_INFO
{
    unsigned int   block_size;
    unsigned int   max_blk_num;
    unsigned int   free_blk_num;
}BLK_INFO;

typedef struct MEM_USE_INFO
{
    unsigned int   block_type;
    BLK_INFO      *block_info;
}MEM_USE_INFO;

typedef struct MEM_REGION
{
    unsigned int  realloc_flag;          /* realloc flag, if set true, will malloc from sys when the mempool is not enough  */
    unsigned int  check_flag;            /* check the mem flag */
    unsigned int  type_count;           /*  the block type count */
    MEM_ELMNT    *mem_elmnt_info_p;
    MEM_USE_INFO  mem_use_info;
}MEM_REGION;



unsigned int mem_region_init(MEM_REGION *regionInfo, MEM_CFG *MemCfg, unsigned int count,unsigned int realloc_flag,unsigned int check_flag);

void mem_region_deinit(MEM_REGION *regionInfo);

void *core_malloc(MEM_REGION *regionInfo, unsigned int size ,unsigned int *OUT_size);

unsigned int core_free(MEM_REGION *regionInfo, void *mem_free );

#ifdef __cplusplus
#if __cplusplus
}
#endif /* __cpluscplus */
#endif /* __cpluscplus */

#endif

