#include "dis.h"
#include "log.h"
#include "disrequest.h"
#include "disauth.h"
#include "disnetwork.h"
#include "disosadapt.h"
#include "curl/curl.h"
#include "discommon.h"
#include "dismemmgt.h"
#include "discommon.h"
#include "cJSON.h"
#include <string.h>
#include "httpbody.pb-c.h"

CurlCacheList g_CurlCacheList = {0};

int DisCurlCacheInit()
{
    int ret = 0;

    printfdbgfunc;

    if(g_CurlCacheList.InitFlag)
    {
        return DISStatusReInit;
    }

    g_CurlCacheList.InitFlag = 1;
    ret = SmCreate(&g_CurlCacheList.mutex);
    if(0 != ret)
    {
        g_CurlCacheList.InitFlag = 0;
        return ret;
    }

    g_CurlCacheList.CurrentCount = 0;
    APR_RING_INIT(&(g_CurlCacheList.CurlList),  _CurlCacheItem, CurlItem);

	printfdbgoutfunc;

    return 0;
}

int DisCurlCacheDeinit()
{
    CurlCacheItem  *pCur;
    CurlCacheItem  *pNext;
    int count = 0;
    CurlCacheList *list = &g_CurlCacheList;

    printfdbgfunc;

    if(list->InitFlag != 1)
    {
        return 0;
    }

    SmP(&(list->mutex), 0);

    APR_RING_FOREACH_SAFE(pCur, pNext, &(list->CurlList), _CurlCacheItem, CurlItem)
    {
        curl_easy_cleanup(pCur->CurlHandle);
        /* 内存不用释放，直接在memdeinit中进行统一销毁 */
    }
    SmV(&(list->mutex));

    SmDelete(&(list->mutex));

	printfdbgoutfunc;

    return 0;
}

CurlCacheItem *DisGetCurlItem()
{
    CurlCacheItem *item = NULL;
    CurlCacheList *list = &g_CurlCacheList;
    int ret = 0;

    printfdbgfunc;

    if(list->InitFlag != 1)
    {
        return NULL;
    }
    SmP(&(list->mutex), 0);
    if(!(APR_RING_EMPTY(&(list->CurlList), _CurlCacheItem, CurlItem)))
    {

        item = APR_RING_FIRST(&(list->CurlList));
        APR_RING_REMOVE(item, CurlItem);
        APR_RING_ELEM_INIT(item, CurlItem);
        printfdbgfunc;
    }

    
    SmV(&(list->mutex));

    if(NULL == item)
    {
        item = (CurlCacheItem *)disMemAlloc(sizeof(CurlCacheItem));
        if(NULL == item)
        {
            return NULL;
        }
        memset(item, 0 ,sizeof(CurlCacheItem));
        APR_RING_ELEM_INIT(item, CurlItem);

        item->CurlHandle = curl_easy_init();
        if(NULL == item->CurlHandle)
        {
            disMemFree(item);
            return NULL;
        }
    }

	printfdbgoutfunc;

    return item;
}

int CurlCacheReleaseItem(CurlCacheItem *item)
{
    CurlCacheList *list = &g_CurlCacheList;
    int ret = 0;

    printfdbgfunc;

    if(NULL == item)
    {
        return DISStatusInvalidParam;
    }

    if(list->InitFlag != 1)
    {
        return DISStatusNotInit;
    }

    SmP(&(list->mutex), 0);

    APR_RING_INSERT_TAIL(&(list->CurlList), item, _CurlCacheItem, CurlItem);
    list->CurrentCount++;

    SmV(&(list->mutex));

	printfdbgoutfunc;

    return 0;
}

void DisNetworkDeinit()
{

	printfdbgfunc;
    DisCurlCacheDeinit();
    curl_global_cleanup();
    kill_locks();
	printfdbgoutfunc;

    return;
}

DISStatus DisNetworkInit()
{
    int iRet = 0;

    printfdbgfunc;
#if 1
    iRet = curl_global_init(CURL_GLOBAL_ALL);
    if(0 != iRet)
    {
        logPrint(DISLogLevelALL, "curl init error : %d\r\n", iRet);
        DisNetworkDeinit();
        return DISStatusInternalError;
    }

    iRet = init_locks();
    if(0 != iRet)
    {
        logPrint(DISLogLevelALL, "openssl lock init error : %d\r\n", iRet);
        DisNetworkDeinit();
        return DISStatusInternalError;
    }

#endif

    iRet = DisCurlCacheInit();
    if(0 != iRet)
    {
        logPrint(DISLogLevelALL, "Curl cache init error : %d\r\n", iRet);
        DisNetworkDeinit();
        return DISStatusInternalError;
    }

	printfdbgoutfunc;

    return 0;
}



size_t curl_write_func(void *ptr, size_t size, size_t nmemb, void *data)
{
    size_t len = size * nmemb;
    int MemAllocLen = 0;
    DisRequest *request = (DisRequest *)data;
    char *RspBody = NULL;
    char *tempBody = NULL;

    printfdbgfunc;  

    if(len == 0)
    {
        return len;
    }

    //printf("%s", request->RspBody);

    do
    {
        if(NULL == request->RspBody)
        {
            logPrint(DISLogLevelInfo,"the RspBody is NULL, init it, the len is %d, Cal len is %d\r\n", len, request->RspBodyCalLen);
            request->RspBodyCalLen = (request->RspBodyCalLen > len) ? request->RspBodyCalLen : len * 2;
            request->RspBody = (char *)disMemAlloc(request->RspBodyCalLen);
            if(NULL == request->RspBody)
            {
                break;
            }
            memset(request->RspBody, 0, request->RspBodyCalLen);
            request->RspBodyCurrentLen = 0;
        }

        if(len + request->RspBodyCurrentLen >= request->RspBodyCalLen)
        {
            logPrint(DISLogLevelInfo,"buffer is not enough, the current len is  %d, Cal len is %d, len is %d\r\n", request->RspBodyCurrentLen, request->RspBodyCalLen, len);
            MemAllocLen = len * 2 + request->RspBodyCurrentLen; 
            RspBody = (char *)disMemAlloc(MemAllocLen);
            if(NULL == RspBody)
            {
                break;
            }
            memset(RspBody, 0, MemAllocLen);


			memcpy(RspBody, request->RspBody, request->RspBodyCurrentLen);
            disMemFree(request->RspBody);
            request->RspBody = RspBody;
        }

        logPrint(DISLogLevelInfo,"start copy data, the current len is  %d, Cal len is %d, len is %d\r\n", request->RspBodyCurrentLen, request->RspBodyCalLen, len);


		memcpy(&request->RspBody[request->RspBodyCurrentLen], (char *)ptr, len);
        request->RspBodyCurrentLen += len;

        //printf("%s", request->RspBody);
    }while(0);

    printfdbgoutfunc;   

    return len;
}




DISStatus DisComposeUrl(CURL *handle, DisRequest *pstRequest, DISContext  *disContext, char *url, int maxSize)
{
    int len = 0; 
    int loop = 0;
    CommonKeyValue *pTemp = NULL;

    printfdbgfunc;

    if(NULL == handle || NULL == pstRequest || NULL == url)
    {
        return DISStatusInvalidParam;
    }

    STRING_APPEND(url, len, maxSize, "https://%s%s", disContext->hostname, pstRequest->request.uri);

    if(0 != pstRequest->request.queryParamList.itemCount)
    {
        STRING_APPEND(url, len, maxSize, "%s", "?");
    }


    pTemp = (CommonKeyValue *)pstRequest->request.queryParamList.pItem;
    for(loop = 0; loop < pstRequest->request.queryParamList.itemCount && NULL != pTemp; loop++)
    {
        if(loop != 0)
        {
            STRING_APPEND(url, len, maxSize, "%s", "&");
        }

        STRING_APPEND(url, len, maxSize, "%s", pTemp->valueContent);
        pTemp = pTemp->next;
    }

	printfdbgoutfunc;

    return 0;
}

DISStatus DisSetupUrl(CURL *handle, DisRequest *pstRequest, char *url, char *xSecurityToken)
{
    int loop = 0;
    int len = 0;
    char ContentLenHeader[128] = {0};
    CommonKeyValue *pTemp = NULL;
    CURLcode status = CURLE_OK;

    printfdbgfunc;

    #define curl_easy_setopt_safe(opt, val)                                 \
    if ((status = curl_easy_setopt                                      \
         (handle, opt, val)) != CURLE_OK) {                      \
        return DISStatusFailedToInitializeRequest;                       \
    }

    // Set write callback and data
    curl_easy_setopt_safe(CURLOPT_WRITEFUNCTION, &curl_write_func);
    curl_easy_setopt_safe(CURLOPT_WRITEDATA, pstRequest);

    curl_easy_setopt_safe(CURLOPT_NOSIGNAL, 1);
    curl_easy_setopt_safe(CURLOPT_TIMEOUT , 300L);

    // Turn off Curl's built-in progress meter
    curl_easy_setopt_safe(CURLOPT_NOPROGRESS, 1);

    curl_easy_setopt_safe(CURLOPT_VERBOSE, 1);

    curl_easy_setopt_safe(CURLOPT_SSL_VERIFYPEER, 0);
    
    curl_easy_setopt_safe(CURLOPT_SSL_VERIFYHOST, 0);

    printf("the header count is %d\r\n",pstRequest->request.httpHeaderList.itemCount);
    pTemp = (CommonKeyValue *)pstRequest->request.httpHeaderList.pItem;
    for (loop = 0; loop < pstRequest->request.httpHeaderList.itemCount && (NULL != pTemp); loop++) 
    {
        printf("add header is: %s\r\n",pTemp->valueContent);
        pstRequest->headers = curl_slist_append(pstRequest->headers, pTemp->valueContent);
        pTemp = pTemp->next;
    }

    //curl_slist_append(pstRequest->headers, "Content-Type: application/json");

    if(0 != strlen(xSecurityToken))
    {
        printf("the token is %s \r\n", xSecurityToken);
        curl_slist_append(pstRequest->headers, xSecurityToken);
    }

    //STRING_APPEND(ContentLenHeader, len, 128, "Content-Length: %d", pstRequest->request.bodyCurrentLen);
    //curl_slist_append(pstRequest->headers, ContentLenHeader);
    curl_slist_append(pstRequest->headers, pstRequest->request.authString);

    // Set the HTTP headers
    curl_easy_setopt_safe(CURLOPT_HTTPHEADER, pstRequest->headers);

    // Set URI
    curl_easy_setopt_safe(CURLOPT_URL, url);

    switch(pstRequest->msgType)
    {
        case DISInterfaceCreateStream:
        case DISInterfaceCreateApp:
        case DISInterfacePutRecords:
        case DISInterfaceCommitCheckpoint:
            curl_easy_setopt_safe(CURLOPT_POST, 1L);
            curl_easy_setopt_safe(CURLOPT_POSTFIELDS, (pstRequest->request.body));
            break;
        case DISInterfaceDeleteStream:
        case DISInterfaceDeleteApp:
            curl_easy_setopt_safe(CURLOPT_CUSTOMREQUEST, "DELETE");
            break;
        case DISInterfaceListStream:
        case DISInterfaceDescribeStream:
        case DISInterfaceGetCursor:
        case DISInterfaceGetRecords:
        case DISInterfaceGetCheckpoint:
        default: 
            break;
    }

	printfdbgoutfunc;

    return 0;
}

DISStatus DisGetCurlErrCode(CURLcode curlCode)
{

    printfdbgfunc;

    switch(curlCode) 
    {
    case CURLE_OUT_OF_MEMORY:
        return DISStatusNoMemory;
    case CURLE_COULDNT_RESOLVE_HOST:
    case CURLE_COULDNT_RESOLVE_PROXY:
        return DISStatusNameLookupError;
    case CURLE_COULDNT_CONNECT:
        return DISStatusFailToConnect;
    case CURLE_OPERATION_TIMEDOUT:
        return DISStatusOpTimeOut;
    default: 
        return DISStatusInternalError;
    }
}

DISStatus   DisSendRequest(DisRequest *pstRequest, DISContext  *disContext, DISResponseInfo *ResponseInfo, char *xSecurityToken)
{
    DISStatus ret;
    CURL *handle = NULL;
    CURLcode curlCode = 0;
    char rawUrl[1024]  = {0};
    cJSON *ErrCode = NULL;
    cJSON *ErrMessage = NULL;
    cJSON *ErrorBody = NULL;

    CurlCacheItem *CurlItem = NULL;

    printfdbgfunc;

    ret = compose_auth_header(&(pstRequest->request), disContext->ak, disContext->sk, disContext->region, "dis");
    if(0 != ret)
    {
        return ret;
    }


    CurlItem = DisGetCurlItem();
    if(NULL == CurlItem)
    {
        return DISStatusNoMemory;
    }
    handle = CurlItem->CurlHandle;
    curl_easy_reset(handle);

    do
    {
        ret = DisComposeUrl(handle, pstRequest, disContext, rawUrl, sizeof(rawUrl));
        if(0 != ret)
        {
            break;
        }

        ret = DisSetupUrl(handle, pstRequest, rawUrl, xSecurityToken);
        if(0 != ret)
        {
            break;
        }

        curlCode = curl_easy_perform(handle);
        if(CURLE_OK != curlCode)
        {
            ret = DisGetCurlErrCode(curlCode);
            break;
        }

        (void)curl_easy_getinfo(handle, CURLINFO_RESPONSE_CODE, &(pstRequest->httpResponseCode));
        logPrint(DISLogLevelInfo,"the rspcode is %d\r\n", pstRequest->httpResponseCode);
        
		int serrializeMode = DISGetSerializedMode();
        ResponseInfo->HttpResponseCode = pstRequest->httpResponseCode;
        //request failed
		if ((ResponseInfo->HttpResponseCode >= 300 || ResponseInfo->HttpResponseCode < 200) && PROTOBUFFMODE == serrializeMode)
		{
			//failed, pstRequest->RspBody content is $2 = 0x743150 "{\"errorCode\":\"DIS.4302\",\"message\":\"Partition does not exist. [dis-shawn-1][0][c28539c6e40d4bf3adc70bb36678864a]\"}"
			if (DISInterfaceGetRecords == pstRequest->msgType)
			{
				GetRecordsResult *pstGetRecordsResult;
				pstGetRecordsResult = get_records_result__unpack(NULL, strlen(pstRequest->RspBody), pstRequest->RspBody);
				if (NULL == pstGetRecordsResult)
				{
					printf("%s:%d get_records_result__unpack failed\n", __FUNCTION__, __LINE__);
					return DISStatusError;
				}
			}
			else if (DISInterfacePutRecords == pstRequest->msgType)
			{
				PutRecordsResult *pstPutRecordsResult;
				pstPutRecordsResult = put_records_result__unpack(NULL, strlen(pstRequest->RspBody), pstRequest->RspBody);
				if (!pstPutRecordsResult)
				{
					printf("%s:%d put_records_result__unpack failed\n", __FUNCTION__, __LINE__);
					return DISStatusError;
				}
			}
			else
			{
				logPrint(DISLogLevelInfo, "Error %d, start process the body: %s\r\n", ResponseInfo->HttpResponseCode, pstRequest->RspBody);
				ErrorBody = cJSON_Parse(pstRequest->RspBody);
				if (NULL == ErrorBody)
				{
					ret = DISStatusInvalidResponse;
					break;
				}
				pstRequest->jsonData = ErrorBody;

				ErrCode = cJSON_GetObjectItem(ErrorBody, "errorCode");
				ErrMessage = cJSON_GetObjectItem(ErrorBody, "message");

				if (NULL == ErrCode || NULL == ErrCode->valuestring || NULL == ErrMessage || NULL == ErrMessage->valuestring)
				{
					ret = DISStatusInvalidResponse;
					break;
				}

				strncpy(ResponseInfo->ErrorCode, ErrCode->valuestring, strlen(ErrCode->valuestring));
				strncpy(ResponseInfo->ErrorDetail, ErrMessage->valuestring, strlen(ErrMessage->valuestring));

				ret = DISStatusResponseError;
			}
        }
		else if ((ResponseInfo->HttpResponseCode >= 300 || ResponseInfo->HttpResponseCode < 200) && BASE64MODE == serrializeMode)
		{
			logPrint(DISLogLevelInfo, "Error %d, start process the body: %s\r\n", ResponseInfo->HttpResponseCode, pstRequest->RspBody);
			ErrorBody = cJSON_Parse(pstRequest->RspBody);
			if (NULL == ErrorBody)
			{
				ret = DISStatusInvalidResponse;
				break;
			}
			pstRequest->jsonData = ErrorBody;

			ErrCode = cJSON_GetObjectItem(ErrorBody, "errorCode");
			ErrMessage = cJSON_GetObjectItem(ErrorBody, "message");

			if (NULL == ErrCode || NULL == ErrCode->valuestring || NULL == ErrMessage || NULL == ErrMessage->valuestring)
			{
				ret = DISStatusInvalidResponse;
				break;
			}

			strncpy(ResponseInfo->ErrorCode, ErrCode->valuestring, strlen(ErrCode->valuestring));
			strncpy(ResponseInfo->ErrorDetail, ErrMessage->valuestring, strlen(ErrMessage->valuestring));

			ret = DISStatusResponseError;
		}

    }while(0);

    CurlCacheReleaseItem(CurlItem);

	printfdbgoutfunc;

    return ret;
}