#include "stdio.h"
#include "time.h"
#include "log.h"
#include "dis.h"
#include "disauth.h"
#include "disrequest.h"
#include "disbodyencode.h"
#include "dismemmgt.h"
#include "discommon.h"
#include "curl/curl.h"
#include "cJSON.h"

#define DEFAULTHEADERCOUNT 4

// Convert an HttpRequestType to an HTTP Verb string
char *DisGetMethodByMsgType(DISInterfaceType requestType)
{
    printfdbgfunc;

    switch (requestType) {
    case DISInterfaceCreateStream:
    case DISInterfaceCreateApp:
    case DISInterfacePutRecords:
    case DISInterfaceCommitCheckpoint:
        return "POST";
    case DISInterfaceDeleteStream:
    case DISInterfaceDeleteApp:
        return "DELETE";
    case DISInterfaceListStream:
    case DISInterfaceDescribeStream:
    case DISInterfaceGetCursor:
    case DISInterfaceGetRecords:
    case DISInterfaceGetCheckpoint:
    default: 
        return "GET";
    }
}

int DisCalRspBodyLen(DISInterfaceType requestType)
{
    printfdbgfunc;

    switch (requestType) {
    case DISInterfacePutRecords:
        return 0;
    case DISInterfaceListStream:
        return BASE_RSP_BODY_LEN;
    case DISInterfaceDescribeStream:
        return BASE_RSP_BODY_LEN + (160 * 200);
    case DISInterfaceGetCursor:
        return BASE_RSP_BODY_LEN;
    case DISInterfaceGetRecords:
        return MAX_RSP_BODY_LEN;
    case DISInterfaceGetCheckpoint:
        return BASE_RSP_BODY_LEN;

    case DISInterfaceCreateStream:
    case DISInterfaceCreateApp: 
    case DISInterfaceCommitCheckpoint:
    case DISInterfaceDeleteStream:
    case DISInterfaceDeleteApp:
    default: 
        return BASE_RSP_BODY_LEN;
    }
}

int DisCalculateRequestSize(DISInterfaceType requestType, int additionalSize)
{
    return DIS_REQ_BASE_SIZE + additionalSize;
}

void * DisRequestAlloc(DisRequest *disRequest, int size)
{
    void *temp = NULL;
    int tempSize = 0;

    printfdbgfunc;

    if(NULL == disRequest || 0 >= size)
    {
        return NULL;
    }

    tempSize = CORE_ALIGN(size);

    if(disRequest->MaxSize <= disRequest->CurrentSize + tempSize)
    {
        return NULL;
    }

    temp = disRequest->FreeMemPtr;

    disRequest->FreeMemPtr = disRequest->FreeMemPtr + tempSize;
    disRequest->CurrentSize += tempSize;

	printfdbgoutfunc;

    return temp;
}



DISStatus DisValidateDisString(const char *string, int len, int maxLen)
{
    int loop = 0;

    printfdbgfunc;

    if(len > maxLen)
    {
        return DISStatusInvalidParam;
    }

    for(loop = 0; loop < len; loop++)
    {
        if((string[loop] >= 'a' && string[loop] <= 'z') || (string[loop] >= 'A' && string[loop] <= 'Z') || (string[loop] >= '0' && string[loop] <= '9') || (string[loop] == '-') || (string[loop] == '_'))
        {
            continue;
        }
        return DISStatusInvalidParam;
    }

	printfdbgoutfunc;

    return DISStatusOK;
}

DISStatus DisComposeUri(DisRequest *disRequest, DISContext *context, char *externData)
{
    int len = 0;
    int loop = 0;
    int maxLen = MAX_URI_SIZE;
    char *pTemp = NULL;
    char *UriStart = NULL;
    char *QueryStringStart = NULL;
    CommonRequest *pCommonReq = NULL;

    printfdbgfunc;

    pCommonReq = &(disRequest->request);

    
    pTemp = DisRequestAlloc(disRequest, 256);
    if(NULL == pTemp)
    {
        return DISStatusNoMemory;
    }
    
    pCommonReq->uri = pTemp;

    len = sprintf(pTemp, "/v2/%s/", context->projectId);
    pTemp += len;

    switch(disRequest->msgType)
    {
        case   DISInterfaceCreateStream:
            len = sprintf(pTemp, "streams/");
            pTemp += len;
            break;

        case   DISInterfaceDeleteStream:
            len = sprintf(pTemp, "streams/%s/", externData);
            pTemp += len;
            break;

        case   DISInterfaceListStream:      
            len = sprintf(pTemp, "streams/");
            pTemp += len;
            break;

        case   DISInterfaceDescribeStream:
            len = sprintf(pTemp, "streams/%s/", externData);
            pTemp += len;
            break;

        case   DISInterfacePutRecords:
            len = sprintf(pTemp, "records/");
            pTemp += len;
            break;

        case   DISInterfaceGetCursor:
            len = sprintf(pTemp, "cursors/");
            pTemp += len;
            break;

        case   DISInterfaceGetRecords:
            len = sprintf(pTemp, "records/");
            pTemp += len;
            break;

        case   DISInterfaceCreateApp:
            len = sprintf(pTemp, "apps/");
            pTemp += len;
            break;

        case   DISInterfaceDeleteApp:
            len = sprintf(pTemp, "apps/%s/", externData);
            pTemp += len;
            break;

        case   DISInterfaceCommitCheckpoint:
            len = sprintf(pTemp, "checkpoints/");
            pTemp += len;
            break;

        case   DISInterfaceGetCheckpoint:
            len = sprintf(pTemp, "checkpoints/");
            pTemp += len;
            break;

        default:
            break;
    }

	printfdbgoutfunc;
    return DISStatusOK;
}

char * DisRequestAddString(DisRequest *req, char *pKey, char *pValue, KeyType type)
{
    int size = 0;
    int len = 0;
    char *pStr = NULL;
    char jionChar;

    printfdbgfunc;

    if(NULL == req || NULL == pKey || NULL == pValue || KeyTypeAll <= type)
    {
        return NULL;
    }
    jionChar = (type == KeyTypeHeader)?':':'=';

    size = strlen(pKey) + 1 + strlen(pValue) + 2;
    pStr = (char *)DisRequestAlloc(req, size);
    if(NULL == pStr)
    {
        return NULL;
    }

    STRING_APPEND(pStr, len, size, "%s%c%s", pKey, jionChar, pValue);

	printfdbgoutfunc;
    return pStr;
}


DISStatus DisComposeDefaultHeaders(DisRequest *req, DISContext *context,DISInterfaceType msgType)
{
    CommonRequest *pCommonReq = NULL;
    struct tm *flag = NULL;
    char date[64] = {0};
    CommonKeyValue *pDefaultHeaderList = NULL;

    CommonKeyValue *pContentType;
    CommonKeyValue *pHost;
    CommonKeyValue *pxsdkdate;
	CommonKeyValue *pxsdkversion;
	char *versionInfo = g_Version;

    printfdbgfunc;


    pCommonReq = &(req->request);
    pCommonReq->now = time(NULL);
    flag = gmtime(&(pCommonReq->now));
    if(flag != NULL){ 
        strftime(date, sizeof(date), "%Y%m%dT%H%M%SZ", flag);
    }

    pDefaultHeaderList = DisRequestAlloc(req, (sizeof(CommonKeyValue) * DEFAULTHEADERCOUNT));
    if(NULL == pDefaultHeaderList)
    {
        return DISStatusNoMemory;
    }

    pContentType = &(pDefaultHeaderList[0]);
    pHost = &(pDefaultHeaderList[1]);
    pxsdkdate = &(pDefaultHeaderList[2]);
	pxsdkversion = &(pDefaultHeaderList[3]);

    strncpy(pContentType->KeyName, "content-type", strlen("content-type"));
    strncpy(pHost->KeyName, "host", strlen("host"));
    strncpy(pxsdkdate->KeyName, "x-sdk-date", strlen("x-sdk-date"));
	strncpy(pxsdkversion->KeyName, "x-sdk-version", strlen("x-sdk-version"));

    int serrializedMode = DISGetSerializedMode();
    if(PROTOBUFFMODE == serrializedMode && (DISInterfacePutRecords == msgType || DISInterfaceGetRecords == msgType))
    {
        pContentType->valueContent = DisRequestAddString(req, pContentType->KeyName, "application/x-protobuf",  KeyTypeHeader);
    }
    else
    {
        pContentType->valueContent = DisRequestAddString(req, pContentType->KeyName, "application/json",  KeyTypeHeader);
    }
    pHost->valueContent = DisRequestAddString(req, pHost->KeyName, context->hostname, KeyTypeHeader);
    pxsdkdate->valueContent = DisRequestAddString(req, pxsdkdate->KeyName, date, KeyTypeHeader);
	pxsdkversion->valueContent = DisRequestAddString(req, pxsdkversion->KeyName, versionInfo, KeyTypeHeader);

    if((NULL == pContentType->valueContent) || (NULL == pHost->valueContent) || (NULL == pxsdkdate->valueContent) || (NULL == pxsdkversion->valueContent))
    {
        return DISStatusNoMemory;
    }

	printfdbgoutfunc;

    return (insertItemSortByKey(&(pCommonReq->httpHeaderList), pContentType, 1) | insertItemSortByKey(&(pCommonReq->httpHeaderList), pHost, 1) | insertItemSortByKey(&(pCommonReq->httpHeaderList), pxsdkdate, 1) | insertItemSortByKey(&(pCommonReq->httpHeaderList), pxsdkversion, 1));
}

DISStatus DisAddQueryList(DisRequest *req, char *QueryName, char *QueryValue)
{
    CommonKeyValue *pQueryParam = NULL;

    printfdbgfunc;

    if(NULL == req || NULL == QueryName || NULL == QueryValue)
    {
        return DISStatusInvalidParam;
    }

    pQueryParam = DisRequestAlloc(req, (sizeof(CommonKeyValue)));
    if(NULL == pQueryParam )
    {
        return DISStatusNoMemory;
    }

    strncpy(pQueryParam->KeyName, QueryName, strlen(QueryName));

    pQueryParam->valueContent = DisRequestAddString(req, QueryName, QueryValue, KeyTypeQueryParam);
    if(NULL == pQueryParam->valueContent)
    {
        return DISStatusNoMemory;
    }
    
	printfdbgoutfunc;

    return insertItemSortByKey(&(req->request.queryParamList), pQueryParam, 0);
}

DISStatus DisGetRequest(DisRequest **request, DISInterfaceType msgType, DISContext *context, int bodyLen)
{
    DisRequest  *pTempRequest = NULL;
    int size = 0;
    char *pMethod;

    printfdbgfunc;

    if(msgType >= DISInterfaceAll)
    {
        return DISStatusMsgTypeInvalid;
    }

    size = (bodyLen <= 1024) ? DIS_REQ_BASE_SIZE : (DIS_REQ_BASE_SIZE+bodyLen);

    pTempRequest = disMemAlloc(size);
    if(NULL == pTempRequest)
    {
        return DISStatusNoMemory;
    }

    memset(pTempRequest, 0, size);

    pTempRequest->msgType = msgType;
    pTempRequest->FreeMemPtr = (char *)(pTempRequest + sizeof(pTempRequest));
    pTempRequest->MaxSize = size;
    pTempRequest->CurrentSize = sizeof(pTempRequest);

    pTempRequest->RspBodyCalLen = DisCalRspBodyLen(msgType);

    pMethod = DisGetMethodByMsgType(msgType);
    logPrint(DISLogLevelInfo,"the method is %s\r\n", pMethod);
    strncpy(pTempRequest->request.method, pMethod, strlen(pMethod));

    DisComposeDefaultHeaders(pTempRequest, context, msgType);

    if(0 != bodyLen)
    {
        pTempRequest->request.body = DisRequestAlloc(pTempRequest, bodyLen);
        if(NULL == pTempRequest->request.body)
        {
            DisReleaseRequest(pTempRequest);
            return DISStatusNoMemory;
        }
        pTempRequest->request.bodyMaxLen = bodyLen;
    }

    *request = pTempRequest;

	printfdbgoutfunc;

    return DISStatusOK;

}

void DisReleaseRequest(DisRequest *request)
{
    printfdbgfunc;

    if(NULL == request)
    {
        return ;
    }

    if(NULL != request->headers)
    {
        curl_slist_free_all(request->headers);
    }

    if(NULL != request->RspBody)
    {
        disMemFree(request->RspBody);
    }

    if(NULL != request->jsonData)
    {
        cJSON_Delete(request->jsonData);
    }

    disMemFree(request);

    printfdbgoutfunc;

    return;
}

