#include "dis.h"
#include "disosadapt.h"
#include "discommon.h"
#include "dissendtask.h"
#include "log.h"
#include "curl/curl.h"
#include "apr_ring.h"
#include "disrequest.h"
#include "disbodyencode.h"
#include "util.h"
#include "disnetwork.h"
#include "cJSON.h"
#include "disosadapt.h"


int volatile TaskRunFlag = 0;

int DisSendTaskInit(unsigned int *taskId)
{
    unsigned int TempTask = 0;
    int ret = 0;
    CURL *handle = NULL;

    TaskRunFlag = 1;


    ret = CreateTask(DisSendTaskProcess, handle, &TempTask);
    if(0 != ret || 0 == TempTask)
    {
        logPrint(DISLogLevelALL, "Start Task error : %d\r\n", ret);
        return ret;
    }

    *taskId = TempTask;

    return 0;
}

int DisSendTaskDeinit(unsigned int taskId)
{
    unsigned int TempTask = 0;
    int ret = 0;

    TaskRunFlag = 0;

    DisSleep(10);

    DeleteTask(taskId);
    
    return 0;
}

int DisSendTaskBodyEncodeRecords(char *streamName, SendTaskList *SendList, char *data, int *bodyLen, int bodyMaxLen)
{
    int    len = 0, loop = 0, temp = 0;
    char  *pCurrent = NULL;
    DisRecordListItem  *tempItem = NULL;

    printfdbgfunc;

    if(NULL == data || NULL == bodyLen || NULL == streamName || NULL == SendList || 0 == SendList->Count)
    {
        return -1;
    }

    if(APR_RING_EMPTY(&(SendList->SendListHeader), _DisRecordListItem, RecordListLink))
    {
        return -1;
    }

    pCurrent = data;

    DIS_APPEND_CHAR(pCurrent, '{');

    len = sprintf(pCurrent, "\"stream_name\": \"%s\",\"records\":[", streamName);
    pCurrent += len;

    
    int serrializedMode = DISGetSerializedMode();
    for(loop = 0, tempItem = APR_RING_FIRST(&(SendList->SendListHeader)); loop < SendList->Count; loop++, serrializedMode = 0xf0 ,tempItem = APR_RING_NEXT(tempItem, RecordListLink))
    {
        if(0 != loop)
        {
            DIS_APPEND_CHAR(pCurrent, ',');
        }
        //tempRecord = &(record[loop]);

        DIS_APPEND_CHAR(pCurrent, '{');
        DIS_APPEND_STRING(pCurrent, 8, "\"data\":\"", temp);

        STRING_APPEND(pCurrent, len, bodyMaxLen, "%s", "{\"data\":\"");

        if(BASE64MODE == serrializedMode)
        {
            len += base64_encode((unsigned char *)(tempItem->RawData.data), (unsigned char *)&(pCurrent[len]), tempItem->RawData.stringLen);
        }
        else if(PROTOBUFFMODE == serrializedMode)
        {
		    //len += protobuf_encode((unsigned char *)(tempItem->RawData.data), (unsigned char *)&(pCurrent[len]), tempItem->RawData.stringLen);
        }
        else
        {
            logPrint(DISLogLevelError,"body serialized mode wrong ,mode = %d", serrializedMode);
        }

        STRING_APPEND(pCurrent, len,  bodyMaxLen,"\",\"partition_key\": \"%s\"", tempItem->partitionKey);

        if(0 != tempItem->explicitHashKey[0])
        {
            STRING_APPEND(pCurrent, len, bodyMaxLen, ",\"explicit_hash_key\": \"%s\"", tempItem->explicitHashKey);
        }

        if(0 != tempItem->partitionId[0])
        {
            STRING_APPEND(pCurrent, len, bodyMaxLen, ",\"partition_id\": \"%s\"", tempItem->partitionId);
        }

        STRING_APPEND(pCurrent, len, bodyMaxLen, "%s", "}");
    }
    STRING_APPEND(pCurrent, len, bodyMaxLen, "%s", "]}");

    if(BASE64MODE == serrializedMode)
    {
        *bodyLen = len;
    }
    else if(PROTOBUFFMODE == serrializedMode)
    {
        //*bodyLen = protobuf_encode(data, data, len);
    }

    //*bodyLen = len;

	printfdbgoutfunc;

    return 0;
}

void DisReleaseRecordList(SendTaskList  *SendList, DisRecordList *RecordCacheList)
{
    DisRecordListItem *node = NULL;
    DisRecordListItem *tempnode = NULL;
    int Count = 0;

	printfdbgfunc;

    if(NULL == SendList || NULL == RecordCacheList)
    {
        return;
    }

    if(APR_RING_EMPTY(&(SendList->SendListHeader), _DisRecordListItem, RecordListLink))
    {
        return;
    }

    node = APR_RING_LAST(&(SendList->SendListHeader));

    for(Count = 0; Count < SendList->Count && node != NULL; Count++)
    {
        tempnode = node->RecordListLink.prev;

        if(200 != node->RspCode)
        {
            DisRecordListInsertToHead(RecordCacheList, node);
        }
        else
        {
            DisRecordListItemRelease(node);
        }

        node = tempnode;
    }

	printfdbgoutfunc;
}

void DisSendMsgFromRecordList(SendInfo *SendInfoDetail, DisRecordList *RecordList)
{
    DisRecordListItem *ListNode = NULL;
    int Count = 0;
    int MsgLen = 0;
    int CalBodySize = 0;
    int Ret = 0;
    DisRequest *request = NULL;
    DISContext DisCtx = {0};
    char  ak[64] = {0};
    char  sk[128] = {0};
    char *xSecurityToken = NULL;
    SendTaskList SendList = {0};
    char *body = NULL;
    int RealLen = 0;
    cJSON  *BodyData = NULL;
    cJSON  *Record = NULL;
    cJSON  *RecordArray = NULL;
    int     RecordCount = 0;
    cJSON  *SeqNumber = NULL;
    cJSON  *partitionId = NULL;
    DISPutRecord tempRecord = {0};

	printfdbgfunc;

    APR_RING_INIT(&(SendList.SendListHeader),  _DisRecordListItem, RecordListLink);

    Ret = g_GetUserAuthInfo(SendInfoDetail->ProjectId, ak, sk, xSecurityToken);
    if(0 != Ret)
    {
        return;
    }

    while(SendList.Count < MAX_RECORD_NUM)
    {
        ListNode = DisRecordListGetNode(RecordList);
        if(NULL != ListNode)
        {
            if((ListNode->RawData.stringLen + MsgLen) >MAX_MSG_LEN)
            {
                Ret = DisRecordListInsertToHead(RecordList, ListNode);
                if(0 != Ret)
                {
                    logPrint(DISLogLevelALL, "insert record to list error : %d\r\n", Ret);
                }
                break;
            }
            else
            {
                APR_RING_INSERT_HEAD(&(SendList.SendListHeader), ListNode, _DisRecordListItem, RecordListLink);
                SendList.Count++;
                CalBodySize = CalBodySize + (ListNode->RawData.stringLen * 1.5) + strlen(ListNode->partitionKey) + 128;
            }
        }
        else
        {
            break;
        }
    }

    DisCtx.ak = ak;
    DisCtx.sk = sk;
    DisCtx.hostname = SendInfoDetail->Endpoint;
    DisCtx.projectId = SendInfoDetail->ProjectId;
    DisCtx.region = SendInfoDetail->Region;

    while(0 != Count)
    {
        Ret = DisGetRequest(&request, DISInterfacePutRecords, &DisCtx, CalBodySize);
        if(0 != Ret)
        {
            break;
        }

        request->RspBodyCalLen = SendList.Count * 160 + BASE_RSP_BODY_LEN;

        Ret = DisSendTaskBodyEncodeRecords(SendInfoDetail->StreamName, &SendList, request->request.body, &(request->request.bodyCurrentLen), CalBodySize);
        if(0 != Ret)
        {
            /* �������е���Ϣ��д�������� */
            break;
        }

        Ret = DisComposeUri(request, &DisCtx, SendInfoDetail->StreamName);
        if(DISStatusOK != Ret)
        {
            break;
        }

        Ret = DisSendRequest(request, &DisCtx, NULL, xSecurityToken);
        if(0 != Ret || !(request->httpResponseCode >= 200 && request->httpResponseCode < 300) || NULL == request->RspBody)
        {
            /* ���ݷ���ʧ�ܣ���Ҫ���лָ����������ݶ�ʧ */
            break;
        }

        BodyData = cJSON_Parse(request->RspBody);
        request->jsonData = BodyData;
        if(NULL == BodyData)
        {
            break;
        }

        RecordArray = cJSON_GetObjectItem(BodyData, "records");
        if(NULL == RecordArray)
        {
            /* ��ȡrecordsԪ��ʧ�� */
            break;
        }


        for(RecordCount = 0, ListNode = APR_RING_FIRST(&(SendList.SendListHeader)); RecordCount < SendList.Count && NULL != ListNode; RecordCount++, ListNode = APR_RING_NEXT(ListNode, RecordListLink))
        {
            SeqNumber = NULL;
            partitionId = NULL;
            Record = NULL;

            Record = cJSON_GetArrayItem(RecordArray, RecordCount);
            if(NULL == Record)
            {
                break;
            }

            SeqNumber = cJSON_GetObjectItem(Record, "sequence_number");
            partitionId = cJSON_GetObjectItem(Record, "partition_id");

            if(NULL != SeqNumber && NULL != SeqNumber->valuestring && NULL != partitionId && NULL != partitionId->valuestring)
            {
                strncpy(ListNode->seqNumber, SeqNumber->valuestring, strlen(SeqNumber->valuestring));
                strncpy(ListNode->partitionId, partitionId->valuestring, strlen(partitionId->valuestring));
                ListNode->RspCode = 200;

                tempRecord.recordData.stringLen = ListNode->RawData.stringLen;
                tempRecord.recordData.data      = ListNode->RawData.data;
                tempRecord.partitionKey         = ListNode->partitionKey;
                tempRecord.partitionId          = ListNode->partitionId;
                if(ListNode->pfNotify)
                {
                    ListNode->pfNotify(NULL, NULL,  SendInfoDetail->StreamName, &tempRecord, SeqNumber->valuestring, partitionId->valuestring);
                }
            }

            break;
        }

        DisReleaseRequest(request);

        DisReleaseRecordList(&SendList, RecordList);

    }

	printfdbgoutfunc;
}

void DisSendTaskProcess(void *param)
{
    DisRecordListItem *ListNode = NULL;
    SendInfoHashItem  *HashItem = NULL;
    SendInfoHashTable *pHashTable = &g_stSendInfoTable;
    DisRecordList     *RecordList = NULL;
    int MaxSize = pHashTable->MaxCount;
    int loop;       

    while(TaskRunFlag)
    {
        for(loop = 0; loop < MaxSize; loop++)
        {
            HashItem = &(pHashTable->SendInfoItemList[loop]);
            if(1 == HashItem->bExists)
            {
                RecordList = HashItem->RecordList;

                DisSendMsgFromRecordList(&(HashItem->SendInfoDetail),RecordList);

                DisSleep(10);

            }
            else
            {
                continue;
            }
        }
    }
}